import numpy as np
import os
from qtpy import QtGui, QtCore, QtWidgets
import sharppy.sharptab.params as params
import sharppy.sharptab.winds as winds
import sharppy.sharptab.interp as interp
import sharppy.databases.inset_data as inset_data
from sharppy.sharptab.constants import *

## routine written by Kelton Halbert and Greg Blumberg
## keltonhalbert@ou.edu and wblumberg@ou.edu

__all__ = ['backgroundENS', 'plotENS']

class backgroundENS(QtWidgets.QFrame):
    '''
    Draw the background frame and lines for the Theta-E plot frame
    '''
    def __init__(self):
        super(backgroundENS, self).__init__()
        self.initUI()


    def initUI(self):
        ## window configuration settings,
        ## sich as padding, width, height, and
        ## min/max plot axes
        self.setStyleSheet("QFrame {"
            "  background-color: rgb(0, 0, 0);"
            "  border-width: 1px;"
            "  border-style: solid;"
            "  border-color: #3399CC;}")
        if self.physicalDpiX() > 75:
            fsize = 10
        else:
            fsize = 11
        fsize = np.floor(.055 * self.size().height())
        self.fsize = fsize
        self.plot_font = QtGui.QFont('Helvetica', fsize + 1)
        self.box_font = QtGui.QFont('Helvetica', fsize)
        self.plot_metrics = QtGui.QFontMetrics( self.plot_font )
        self.box_metrics = QtGui.QFontMetrics(self.box_font)
        self.plot_height = self.plot_metrics.xHeight() + 5
        self.box_height = self.box_metrics.xHeight() + 5
        self.lpad = 40; self.rpad = 40.
        self.tpad = fsize * 2 + 5; self.bpad = fsize
        self.wid = self.size().width() - self.rpad
        self.hgt = self.size().height() - self.bpad
        self.tlx = self.rpad; self.tly = self.tpad
        self.brx = self.wid; self.bry = self.hgt
        self.ymax = 3000.; self.ymin = 0.
        self.xmax = 3000.; self.xmin = 0.
        self.plotBitMap = QtGui.QPixmap(self.width()-2, self.height()-2)
        self.plotBitMap.fill(self.bg_color)
        self.plotBackground()

    def resizeEvent(self, e):
        '''
        Handles the event the window is resized
        '''
        self.initUI()
    
    def plotBackground(self):
        '''
        Handles painting the frame.
        '''
        ## initialize a painter object and draw the frame
        qp = QtGui.QPainter()
        qp.begin(self.plotBitMap)
        qp.setRenderHint(qp.Antialiasing)
        qp.setRenderHint(qp.TextAntialiasing)
        self.draw_frame(qp)
        qp.end()

    def setBlackPen(self, qp):
        color = QtGui.QColor('#000000')
        color.setAlphaF(.5)
        pen = QtGui.QPen(color, 0, QtCore.Qt.SolidLine)
        brush = QtGui.QBrush(QtCore.Qt.SolidPattern)
        qp.setPen(pen)
        qp.setBrush(brush)
        return qp

    def draw_frame(self, qp):
        '''
        Draw the background frame.
        qp: QtGui.QPainter object
        '''
        ## set a new pen to draw with
        EF1_color = "#006600"
        EF2_color = "#FFCC33"
        EF3_color = "#FF0000"
        EF4_color = "#FF00FF"

        pen = QtGui.QPen(self.fg_color, 2, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        qp.setFont(self.plot_font)
        rect1 = QtCore.QRectF(1.5, 2, self.brx, self.plot_height)
        qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter,
            'Ensemble Indices')

        pen = QtGui.QPen(QtCore.Qt.blue, 1, QtCore.Qt.DashLine)
        qp.setPen(pen)
        ytick_fontsize = self.fsize-1
        y_ticks_font = QtGui.QFont('Helvetica', ytick_fontsize)
        qp.setFont(y_ticks_font)
        efstp_inset_data = inset_data.condSTPData()
        #texts = efstp_inset_data['ytexts']
        spacing = self.bry / 10.
        texts = ['0','1000','2000','3000']
        y_ticks = [0,1000,2000,3000]#np.arange(self.tpad, self.bry+spacing, spacing)
        for i in range(len(y_ticks)):
            #print y_ticks[i]
            pen = QtGui.QPen(QtGui.QColor("#0080FF"), 1, QtCore.Qt.DashLine)
            qp.setPen(pen)
            try:
                qp.drawLine(self.tlx, self.y_to_ypix(int(texts[i])), self.brx, self.y_to_ypix(int(texts[i])))
            except:
                continue
            color = QtGui.QColor('#000000')
            pen = QtGui.QPen(color, 1, QtCore.Qt.SolidLine)
            qp.setPen(pen)
            ypos = spacing*(i+1) - (spacing/4.)
            ypos = self.y_to_ypix(int(texts[i])) - ytick_fontsize/2
            xpos = self.tlx - 50/2.
            rect = QtCore.QRect(xpos, ypos, 50, ytick_fontsize)
            pen = QtGui.QPen(self.fg_color, 1, QtCore.Qt.SolidLine)
            qp.setPen(pen)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter, texts[i])

        width = self.brx / 12
        spacing = self.brx / 12

        # Draw the x tick marks
        qp.setFont(QtGui.QFont('Helvetica', self.fsize-1))
        for i in range(np.asarray(texts).shape[0]):
            pen = QtGui.QPen(QtGui.QColor("#0080FF"), 1, QtCore.Qt.DashLine)
            qp.setPen(pen)
            try:
                qp.drawLine(self.x_to_xpix(int(texts[i])), self.tly,  self.x_to_xpix(int(texts[i])),self.bry)
            except:
                continue
            color = QtGui.QColor('#000000')
            color.setAlpha(0)
            pen = QtGui.QPen(color, 1, QtCore.Qt.SolidLine)
            ypos = self.y_to_ypix(0)
            xpos = self.x_to_xpix(float(texts[i])) - 50 / 2.
            rect = QtCore.QRect(xpos, ypos, 50, ytick_fontsize)
            # Change to a white pen to draw the text below the box and whisker plot
            pen = QtGui.QPen(self.fg_color, 1, QtCore.Qt.SolidLine)
            qp.setPen(pen)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter, texts[i])


    def y_to_ypix(self, y):
        scl1 = self.ymax - self.ymin
        scl2 = self.ymin + y
        #print scl1, scl2, self.bry, self.tpad, self.tly
        return self.bry - (scl2 / scl1) * (self.bry - self.tpad)

    def x_to_xpix(self, x):
        scl1 = self.xmax - self.xmin
        scl2 = self.xmax - x
        return self.brx - (scl2 / scl1) * (self.brx - self.rpad)


class plotENS(backgroundENS):
    '''
    Plot the data on the frame. Inherits the background class that
    plots the frame.
    '''
    def __init__(self):
        self.bg_color = QtGui.QColor('#000000')
        self.fg_color = QtGui.QColor('#ffffff')

        self.use_left = False

        super(plotENS, self).__init__()
        self.prof = None
        self.pc_idx = 0
        self.prof_collections = []

    def addProfileCollection(self, prof_coll):
        # Add a profile collection to the scatter plot
        self.prof_collections.append(prof_coll)

    def rmProfileCollection(self, prof_coll):
        # Remove a profile collection from the scatter plot
        self.prof_collections.remove(prof_coll)

    def setActiveCollection(self, pc_idx, **kwargs):
        # Set the active profile collection that is being shown in SPCWindow.
        self.pc_idx = pc_idx
        prof = self.prof_collections[pc_idx].getHighlightedProf()

        self.prof = prof
        self.hght = prof.hght

        self.clearData()
        self.plotData()
        self.update()

    def setProf(self, prof):
        # Set the current profile being viewed in the SPC window.
        self.prof = prof

        # Some code to show whether or not the left or right mover is being used.
        #if self.use_left:
        #    self.stpc = prof.left_stp_cin
        #else:
        #    self.stpc = prof.right_stp_cin

        self.clearData()
        self.plotBackground()
        self.plotData()
        self.update()

    def setPreferences(self, update_gui=True, **prefs):
        # Set the preferences for the inset.
        self.bg_color = QtGui.QColor(prefs['bg_color'])
        self.fg_color = QtGui.QColor(prefs['fg_color'])

        if update_gui:
            if self.use_left:
                self.stpc = self.prof.left_stp_cin
            else:
                self.stpc = self.prof.right_stp_cin

            self.clearData()
            self.plotBackground()
            self.plotData()
            self.update()

    def setDeviant(self, deviant):
        # Set the variable to indicate whether or not the right or left mover is being used.
        self.use_left = deviant == 'left'

        if self.use_left:
            self.stpc = self.prof.left_stp_cin
        else:
            self.stpc = self.prof.right_stp_cin

        self.clearData()
        self.plotBackground()
        self.plotData()
        self.update()

    def resizeEvent(self, e):
        '''
        Handles when the window is resized
        '''
        super(plotENS, self).resizeEvent(e)
        self.plotData()
    
    def paintEvent(self, e):
        super(plotENS, self).paintEvent(e)
        qp = QtGui.QPainter()
        qp.begin(self)
        qp.drawPixmap(1, 1, self.plotBitMap)
        qp.end()

    def clearData(self):
        '''
        Handles the clearing of the pixmap
        in the frame.
        '''
        self.plotBitMap = QtGui.QPixmap(self.width(), self.height())
        self.plotBitMap.fill(self.bg_color)
    
    def plotData(self):
        '''
        Handles drawing of data on the frame.
        '''
        if self.prof is None:
            return

        ## this function handles painting the plot
        ## create a new painter obkect
        qp = QtGui.QPainter()
        qp.begin(self.plotBitMap)
        qp.setRenderHint(qp.Antialiasing)
        qp.setRenderHint(qp.TextAntialiasing)

        cur_dt = self.prof_collections[self.pc_idx].getCurrentDate()
        bc_idx = 0
        for idx, prof_coll in enumerate(self.prof_collections):
            # Draw all unhighlighed ensemble members
            if prof_coll.getCurrentDate() == cur_dt:
                proflist = list(prof_coll.getCurrentProfs().values())

                if idx == self.pc_idx:
                    for prof in proflist:
                        self.draw_ensemble_point(qp, prof)
                else:
                    for prof in proflist:
                        self.draw_ensemble_point(qp, prof)
                    #%bc_idx = (bc_idx + 1) % len(self.background_colors)

        bc_idx = 0
        for idx, prof_coll in enumerate(self.prof_collections):
            # Draw all highlighted members that aren't the active one.
            if idx != self.pc_idx and (prof_coll.getCurrentDate() == cur_dt or self.all_observed):
                prof = prof_coll.getHighlightedProf()
                self.draw_ensemble_point(qp, prof)
                #bc_idx = (bc_idx + 1) % len(self.background_colors)

    def draw_ensemble_point(self, qp, prof):
        # Plot the profile index on the scatter plot
        if 'pbl_h' not in dir(prof): # Make sure a PBL top has been found in the profile object
            ppbl_top = params.pbl_top(prof)
            setattr(prof, 'pbl_h', interp.to_agl(prof, interp.hght(prof, ppbl_top)))
        if 'sfcpcl' not in dir(prof): # Make sure a surface parcel has been lifted in the profile object
            setattr(prof, 'sfcpcl', params.parcelx(prof, flag=1 ))
        #x = self.x_to_xpix()
        #y = self.y_to_ypix()
        color = QtCore.Qt.red
        qp.setPen(QtGui.QPen(color))
        qp.setBrush(QtGui.QBrush(color))
        x = self.x_to_xpix(prof.pbl_h) - 50 / 2.
        y = self.y_to_ypix(prof.sfcpcl.bplus) - (self.fsize-1) / 2
        qp.drawEllipse(x, y, 3, 3)

        return


class DrawTest(QtWidgets.QMainWindow):
    def __init__(self, parent=None):
        super(DrawTest, self).__init__(parent)
        # x = np.asarray([1,2,3,4])
        # y = np.asarray([2,2,3,4])
        length = 10
        x = np.random.rand(length) + np.random.randint(0, 10, length)
        y = np.random.rand(length) + np.random.randint(0, 10, length)
        x = np.asarray([0, 5, 10, 0], dtype=float)
        y = np.asarray([0, 5, 10, 20], dtype=float)
        self.frame = plotENS()
        self.frame.addProfileCollection(prof_coll)

        self.frame.setActiveCollection(0)
        self.setCentralWidget(self.frame)

if __name__ == "__main__":
    import sys
    import sharppy.io.buf_decoder as buf_decoder
    path = 'ftp://ftp.meteo.psu.edu/pub/bufkit/SREF/21/sref_oun.buf'
    dec  = buf_decoder.BufDecoder(path)
    prof_coll = dec._parse()
    app = QtGui.QApplication(sys.argv)
    mySW = DrawTest()
    mySW.show()
    sys.exit(app.exec_())



import sharppy.io.spc_decoder as spc_decoder
import sharppy.sharptab.profile as profile
import sharppy.sharptab.watch_type as watch
import numpy.testing as npt
import numpy as np

files = ['examples/data/14061619.OAX']
dec = spc_decoder.SPCDecoder(files[0])
profs = dec.getProfiles()
stn_id = dec.getStnId()

all_profs = profs._profs
prof = all_profs[''][0]
dates = profs._dates
prof = profile.create_profile(pres=prof.pres, hght=prof.hght, tmpc=prof.tmpc, dwpc=prof.dwpc, wspd=prof.wspd, \
                                  wdir=prof.wdir, strictQC=False, profile='convective', date=dates[0])

def test_heat_index():
    temps = np.array([104,100,92,92,86,80,80,60, 30])
    rh = np.array([55,65,60,90,90,75,40, 90, 50])
    correct_hi = np.array([137.361, 135.868, 104.684, 131.256, 105.294, 83.5751, 79.79, 59.965, 30.00])
    returned_hi = []
    for i in range(len(temps)):
        returned_hi.append(watch.heat_index(temps[i], rh[i]))
    returned_hi = np.asarray(returned_hi)
    npt.assert_almost_equal(returned_hi, correct_hi,0) 

def test_wind_chill():
    prof.tmpc[prof.sfc] = 0
    prof.wspd[prof.sfc] = 10
    assert round(watch.wind_chill(prof)) == 23

test_heat_index()
test_wind_chill() 



__author__ = 'keltonhalbert, wblumberg'

import logging
from sharppy.viz import plotSkewT, plotHodo, plotText, plotAnalogues
from sharppy.viz import plotThetae, plotWinds, plotSpeed, plotKinematics #, plotGeneric
from sharppy.viz import plotSlinky, plotWatch, plotAdvection, plotSTP, plotWinter
from sharppy.viz import plotSHIP, plotSTPEF, plotFire, plotVROT
from qtpy.QtCore import *
from qtpy.QtGui import *
from qtpy.QtWidgets import *
import sharppy.sharptab.profile as profile
import sharppy.sharptab as tab
import sharppy.io as io
from sutils.config import Config
from datetime import datetime, timedelta
import numpy as np
import platform
from os.path import expanduser
import os
import re
import logging
#from sharppy._sharppy_version import __version__, __version_name__
from sharppy._version import get_versions
__codename__ = "Andover"
__version__ = get_versions()['version']
del get_versions
__version_name__ = ''

def _modifySheet(sheet, name, value):
    return re.sub("(?<= %s: )#[0-9A-Fa-f]{6}" % name, value, sheet)


class SPCWidget(QWidget):
    """
    This will create the full SPC window, handle the organization
    of the insets, and handle all click/key events and features.
    """

    inset_generators = {
        'SARS':plotAnalogues,
        'STP STATS':plotSTP,
        'COND STP':plotSTPEF,
        'WINTER':plotWinter,
        'FIRE':plotFire,
        'SHIP':plotSHIP,
        'VROT':plotVROT,
    }

    inset_names = {
        'SARS':'Sounding Analogues',
        'STP STATS':'Sig-Tor Stats',
        'COND STP':'EF-Scale Probs (Sig-Tor)',
        'WINTER':'Winter Weather',
        'FIRE':'Fire Weather',
        'SHIP':'Sig-Hail Stats',
        'VROT':'EF-Scale Probs (V-Rot)',
    }

    inset_cfg = {
        ('insets', 'left_inset'):'SARS',
        ('insets', 'right_inset'):'STP STATS',
    }

    parcel_cfg = {
        ('parcel_types', 'pcl1'):'SFC',
        ('parcel_types', 'pcl2'):'ML',
        ('parcel_types', 'pcl3'):'FCST',
        ('parcel_types', 'pcl4'):'MU',
    }

    paths_cfg = {
        ('paths', 'save_img'):expanduser('~'),
        ('paths', 'save_txt'):expanduser('~'),
    }

    def __init__(self, **kwargs):
        parent = kwargs.get('parent', None)

        super(SPCWidget, self).__init__(parent=parent)
        """
        """
        logging.debug("Initializing SPCWidget.")
        ## these are the keyword arguments used to define what
        ## sort of profile is being viewed
        self.prof_collections = []
        self.prof_ids = []
        self.default_prof = None
        self.pc_idx = 0
        self.config = kwargs.get("cfg")
        self.dgz = False
        self.pbl = False
        self.mode = ""

        ## these are used to display profiles
        self.parcel_type = "MU"

        self.coll_observed = False

        self.config.initialize(SPCWidget.inset_cfg)
        self.config.initialize(SPCWidget.parcel_cfg)
        self.config.initialize(SPCWidget.paths_cfg)

        ## these are the boolean flags used throughout the program
        self.swap_inset = False

        ## initialize empty variables to hold objects that will be
        ## used later
        self.left_inset_ob = None
        self.right_inset_ob = None

        ## these are used for insets and inset swapping
        insets = sorted(list(SPCWidget.inset_names.items()), key=lambda i: i[1])
        inset_ids, inset_names = list(zip(*insets))
        self.available_insets = inset_ids
        self.left_inset = self.config['insets', 'left_inset']
        self.right_inset = self.config['insets', 'right_inset']
        self.insets = {}

        self.parcel_types = [self.config['parcel_types', 'pcl1'], self.config['parcel_types', 'pcl2'], \
                             self.config['parcel_types', 'pcl3'], self.config['parcel_types', 'pcl4']]

        ## initialize the rest of the window attributes, layout managers, etc

        self.bg_color = QColor(self.config['preferences', 'bg_color'])
        self.fg_color = QColor(self.config['preferences', 'fg_color'])

        bg_hex = "#%02x%02x%02x" % (self.bg_color.red(), self.bg_color.green(), self.bg_color.blue())
        fg_hex = "#%02x%02x%02x" % (self.fg_color.red(), self.fg_color.green(), self.fg_color.blue())

        self.setStyleSheet("QWidget {background-color: " + bg_hex + ";}")
        ## set the the whole window's layout manager
        self.grid = QGridLayout()
        self.grid.setContentsMargins(1,1,1,1)
        self.grid.setHorizontalSpacing(0)
        self.grid.setVerticalSpacing(2)
        self.setLayout(self.grid)

        ## handle the upper right portion of the window...
        ## hodograph, SRWinds, Storm Slinky, theta-e all go in this frame
        self.urparent = QFrame()
        self.urparent_grid = QGridLayout()
        self.urparent_grid.setContentsMargins(0, 0, 0, 0)
        self.urparent_grid.setVerticalSpacing(0)
        self.urparent.setLayout(self.urparent_grid)
        self.ur = QFrame()
        self.ur.setStyleSheet("QFrame {"
                         "  background-color: " + bg_hex + ";"
                         "  border-width: 0px;"
                         "  border-style: solid;"
                         "  border-color: " + fg_hex + ";"
                         "  margin: 0px;}")

        self.brand = QLabel("SHARPpy v%s %s" % (__version__, __version_name__))
        self.brand.setAlignment(Qt.AlignRight)
        self.brand.setStyleSheet("QFrame {"
                             "  background-color: " + bg_hex + ";"
                             "  text-align: right;"
                             "  padding-top: 4px;"
                             "  padding-bottom: 4px;"
                             "  font-size: 11px;"
                             "  color: " + fg_hex + ";}")

        ## this layout manager will handle the upper right portion of the window
        self.grid2 = QGridLayout()
        self.grid2.setHorizontalSpacing(0)
        self.grid2.setVerticalSpacing(0)
        self.grid2.setContentsMargins(0, 0, 0, 0)
        self.ur.setLayout(self.grid2)
        self.urparent_grid.addWidget(self.brand, 0, 0, 1, 0)
        self.urparent_grid.addWidget(self.ur, 1, 0, 50, 0)
        ## add the upper-right frame to the main frame
        self.grid.addWidget(self.urparent, 0, 1, 3, 1)

        ## Handle the Text Areas
        self.text = QFrame()
        self.text.setStyleSheet("QWidget {"
                            "  background-color: " + bg_hex + ";"
                            "  border-width: 2px;"
                            "  border-style: solid;"
                            "  border-color: #3399CC;}")
        self.grid3 = QGridLayout()
        self.grid3.setHorizontalSpacing(0)
        self.grid3.setContentsMargins(0, 0, 0, 0)
        self.text.setLayout(self.grid3)

        ## set to menu stuff
        self.setUpdatesEnabled(True)
        self.setContextMenuPolicy(Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(self.showCursorMenu)

        ## initialize the data frames
        self.initData()
        self.loadWidgets()

    def getParcelObj(self, prof, name):
        if name == "SFC":
            return prof.sfcpcl
        elif name == "ML":
            return prof.mlpcl
        elif name == "FCST":
            return prof.fcstpcl
        elif name == "MU":
            return prof.mupcl
        elif name == 'EFF':
            return prof.effpcl
        elif name == "USER":
            return prof.usrpcl

    def getParcelName(self, prof, pcl):
        if pcl == prof.sfcpcl:
            return "SFC"
        elif pcl == prof.mlpcl:
            return "ML"
        elif pcl == prof.fcstpcl:
            return "FCST"
        elif pcl == prof.mupcl:
            return "MU"
        elif pcl == prof.effpcl:
            return "EFF"
        elif pcl == prof.usrpcl:
            return "USER"

    def saveimage(self):
        logging.debug("Save an image of the SPCWindow to the disk.")
        path = self.config['paths', 'save_img']
        file_types = "PNG (*.png)"
        file_name, result = QFileDialog.getSaveFileName(self, "Save Image", path, file_types)
        if result:
            self.pixmapToFile(file_name)
            self.config['paths', 'save_img'] = os.path.dirname(file_name)

    def pixmapToFile(self, file_name):
        fmt = file_name.split(".")[-1].upper()
        self.grab().save(file_name, fmt, 100)

    def savetext(self):
        logging.debug("Save the data to the disk.")
        path = self.config['paths', 'save_txt']
        file_types = "TXT (*.txt)"
        file_name, result = QFileDialog.getSaveFileName(self, "Save Sounding Text", path, file_types)
        if result:
            self.default_prof.toFile(file_name)
            self.config['paths', 'save_txt'] = os.path.dirname(file_name)

    def initData(self):
        """
        Initializes all the widgets for the window.
        This gets initially called by __init__
        :return:
        """
        logging.debug("Initializing all widgets in the SPCWindow.")

        self.sound = plotSkewT(dgz=self.dgz, pbl=self.pbl)
#       self.sound.setPreferences(temp_color=temp_color, dewp_color=dewp_color, update_gui=False)
        self.hodo = plotHodo()

        ## initialize the non-swappable insets
        self.speed_vs_height = plotSpeed()
        self.inferred_temp_advection = plotAdvection()
        self.storm_slinky = plotSlinky()
        self.thetae_vs_pressure = plotThetae()
        self.srwinds_vs_height = plotWinds()
        self.watch_type = plotWatch()
        self.convective = plotText(self.parcel_types)
        self.kinematic = plotKinematics()

        # initialize swappable insets
        for inset, inset_gen in SPCWidget.inset_generators.items():
            self.insets[inset] = inset_gen()
#            self.insets[inset].setParent(self.text)

        self.updateConfig(self.config, update_gui=False)

        self.right_inset_ob = self.insets[self.right_inset]
        self.left_inset_ob = self.insets[self.left_inset]

        # Connect signals to slots
        self.convective.updatepcl.connect(self.updateParcel)

        self.sound.parcel.connect(self.defineUserParcel)
        self.sound.modified.connect(self.modifyProf)
        self.sound.reset.connect(self.resetProfModifications)
        self.sound.cursor_toggle.connect(self.hodo.cursorToggle)
        self.sound.cursor_move.connect(self.hodo.cursorMove)

        self.hodo.modified.connect(self.modifyProf)
        self.hodo.modified_vector.connect(self.modifyVector)
        self.hodo.reset.connect(self.resetProfModifications)
        self.hodo.reset_vector.connect(self.resetVector)
        self.hodo.toggle_vector.connect(self.toggleVector)

        self.insets["SARS"].updatematch.connect(self.updateSARS)

    def addProfileCollection(self, prof_col, prof_id, focus=True):
        logging.debug("Adding a Profile Collection to SPCWindow.")

        self.prof_collections.append(prof_col)
        self.prof_ids.append(prof_id)
        self.sound.addProfileCollection(prof_col)
        self.hodo.addProfileCollection(prof_col)

        if focus:
            self.pc_idx = len(self.prof_collections) - 1

        if not prof_col.getMeta('observed'):
            self.coll_observed = False
            self.sound.setAllObserved(self.coll_observed, update_gui=False)
            self.hodo.setAllObserved(self.coll_observed, update_gui=False)

        cur_dt = self.prof_collections[self.pc_idx].getCurrentDate()
        for prof_col in self.prof_collections:
            if not prof_col.getMeta('observed'):
                prof_col.setCurrentDate(cur_dt)

        self.updateProfs()

    @Slot(str)
    def setProfileCollection(self, prof_id):
        logging.debug("Setting the Profile Collection to SPCWindow.")

        try:
            self.pc_idx = self.prof_ids.index(prof_id)
        except ValueError as e:
            logging.exception(e)
            print("Hmmm, that profile doesn't exist to be focused ...")
            return


        cur_dt = self.prof_collections[self.pc_idx].getCurrentDate()
        for prof_col in self.prof_collections:
            if not prof_col.getMeta('observed'):
                prof_col.setCurrentDate(cur_dt)

        self.updateProfs()

    def rmProfileCollection(self, prof_id):
        logging.debug("Removing Profile Collection from SPCWindow.")
        try:
            pc_idx = self.prof_ids.index(prof_id)
        except ValueError as e:
            logging.exception(e)
            print("Hmmm, that profile doesn't exist to be removed ...")
            return

        prof_col = self.prof_collections.pop(pc_idx)
        self.prof_ids.pop(pc_idx)
        self.sound.rmProfileCollection(prof_col)
        self.hodo.rmProfileCollection(prof_col)

        # If we've removed an analog, remove it from the profile it's an analog to.
        if prof_col.hasMeta('filematch'):
            filematch = prof_col.getMeta('filematch')
            for pc in self.prof_collections:
                if pc.hasMeta('analogfile'):
                    keys, vals = list(zip(*list(pc.getMeta('analogfile').items())))
                    if filematch in vals:
                        keys = list(keys); vals = list(vals)

                        idx = vals.index(filematch)
                        vals.pop(idx)
                        keys.pop(idx)

                        pc.setMeta('analogfile', dict(list(zip(keys, vals))))
            self.insets['SARS'].clearSelection()

        if self.pc_idx == pc_idx:
            self.pc_idx = 0
        elif self.pc_idx > pc_idx:
            self.pc_idx -= 1
        self.updateProfs()

    def isAllObserved(self):
        return all( pc.getMeta('observed') for pc in self.prof_collections )

    def isInterpolated(self):
        return self.prof_collections[self.pc_idx].isInterpolated()

    def updateProfs(self):
        logging.debug("Calling SPCWidget.updateProfs")

        prof_col = self.prof_collections[self.pc_idx]
        self.default_prof = prof_col.getHighlightedProf()

        # update the profiles
        self.sound.setActiveCollection(self.pc_idx, update_gui=False)
        self.hodo.setActiveCollection(self.pc_idx)

        self.storm_slinky.setProf(self.default_prof)
        self.inferred_temp_advection.setProf(self.default_prof)
        self.speed_vs_height.setProf(self.default_prof)
        self.srwinds_vs_height.setProf(self.default_prof)
        self.thetae_vs_pressure.setProf(self.default_prof)
        self.watch_type.setProf(self.default_prof)
        self.convective.setProf(self.default_prof)
        self.kinematic.setProf(self.default_prof)

        for inset in list(self.insets.keys()):
            self.insets[inset].setProf(self.default_prof)

        # Update the parcels to match the new profiles
        parcel = self.getParcelObj(self.default_prof, self.parcel_type)
        self.sound.setParcel(parcel)
        self.storm_slinky.setParcel(parcel)

        deviant = 'right' if self.default_prof.latitude >= 0 else 'left'
        self.toggleVector(deviant)

    @Slot(tab.params.Parcel)
    def updateParcel(self, pcl):
        logging.debug("Calling SPCWidget.updateParcel")

        self.parcel_type = self.getParcelName(self.default_prof, pcl)

        self.sound.setParcel(pcl)
        self.storm_slinky.setParcel(pcl)

        self.config['parcel_types', 'pcl1'] = self.convective.pcl_types[0]
        self.config['parcel_types', 'pcl2'] = self.convective.pcl_types[1]
        self.config['parcel_types', 'pcl3'] = self.convective.pcl_types[2]
        self.config['parcel_types', 'pcl4'] = self.convective.pcl_types[3]

    @Slot(str)
    def updateSARS(self, filematch):
        prof_col = self.prof_collections[self.pc_idx]

        dec = io.spc_decoder.SPCDecoder(filematch)
        match_col = dec.getProfiles()

        match_col.setMeta('model', 'Analog')
        match_col.setMeta('run', prof_col.getCurrentDate())
        match_col.setMeta('base_time', prof_col.getCurrentDate())
        match_col.setMeta('observed', True)
        match_col.setMeta('filematch', filematch)
        match_col.setAnalogToDate(prof_col.getCurrentDate())

        dt = prof_col.getCurrentDate()
        if prof_col.hasMeta('analogfile'):
            analogfiles = prof_col.getMeta('analogfile')
            analogfiles[dt] = filematch
        else:
            analogfiles = {dt:filematch}

        prof_col.setMeta('analogfile', analogfiles)

        self.parentWidget().addProfileCollection(match_col, focus=False)

    @Slot(Config)
    def updateConfig(self, config, update_gui=True):
        logging.debug("Updating the SHARPpy GUI configuration.")
        self.config = config
        prefs = dict( (field, value) for (section, field), value in config if section == 'preferences')

        self.bg_color = QColor(prefs['bg_color'])
        self.fg_color = QColor(prefs['fg_color'])

        bg_hex = "#%02x%02x%02x" % (self.bg_color.red(), self.bg_color.green(), self.bg_color.blue())
        fg_hex = "#%02x%02x%02x" % (self.fg_color.red(), self.fg_color.green(), self.fg_color.blue())

        self.sound.setPreferences(update_gui=update_gui, **prefs)
        self.hodo.setPreferences(update_gui=update_gui, **prefs)

        self.storm_slinky.setPreferences(update_gui=update_gui, **prefs)
        self.inferred_temp_advection.setPreferences(update_gui=update_gui, **prefs)
        self.speed_vs_height.setPreferences(update_gui=update_gui, **prefs)
        self.srwinds_vs_height.setPreferences(update_gui=update_gui, **prefs)
        self.thetae_vs_pressure.setPreferences(update_gui=update_gui, **prefs)
        self.watch_type.setPreferences(update_gui=update_gui, **prefs)
        self.convective.setPreferences(update_gui=update_gui, **prefs)
        self.kinematic.setPreferences(update_gui=update_gui, **prefs)

        for inset in list(self.insets.keys()):
            self.insets[inset].setPreferences(update_gui=update_gui, **prefs)

        # Edit style sheets to modify the colors as we need to (surely there's a better way to do this?)
        sheet = self.styleSheet()
        sheet = _modifySheet(sheet, 'background-color', bg_hex)
        self.setStyleSheet(sheet)

        sheet = self.ur.styleSheet()
        sheet = _modifySheet(sheet, 'background-color', bg_hex)
        sheet = _modifySheet(sheet, 'border-color', fg_hex)
        self.ur.setStyleSheet(sheet)

        sheet = self.brand.styleSheet()
        sheet = _modifySheet(sheet, 'background-color', bg_hex)
        sheet = _modifySheet(sheet, 'color', fg_hex)
        self.brand.setStyleSheet(sheet)

        sheet = self.text.styleSheet()
        sheet = _modifySheet(sheet, 'background-color', bg_hex)
        self.text.setStyleSheet(sheet)


    @Slot(tab.params.Parcel)
    def defineUserParcel(self, parcel):
        self.prof_collections[self.pc_idx].defineUserParcel(parcel)
        self.updateProfs()
        self.setFocus()

    @Slot(int, dict)
    def modifyProf(self, idx, kwargs):
        # TODO: Use this as a template to modify layers of the sounding
        self.prof_collections[self.pc_idx].modify(idx, **kwargs)
        self.updateProfs()
        self.setFocus()

    @Slot(str, float, float)
    def modifyVector(self, deviant, vec_u, vec_v):
        self.prof_collections[self.pc_idx].modifyStormMotion(deviant, vec_u, vec_v)
        self.updateProfs()
        self.setFocus()

    @Slot(str)
    def toggleVector(self, deviant):
        self.sound.setDeviant(deviant)
        self.hodo.setDeviant(deviant)

        self.storm_slinky.setDeviant(deviant)
        self.srwinds_vs_height.setDeviant(deviant)
        self.watch_type.setDeviant(deviant)
        self.convective.setDeviant(deviant)
        self.kinematic.setDeviant(deviant)

        self.insets['SARS'].setDeviant(deviant)
        self.insets['STP STATS'].setDeviant(deviant)
        self.insets['COND STP'].setDeviant(deviant)
        self.setFocus()

    def interpProf(self):
        self.prof_collections[self.pc_idx].interp()
        self.updateProfs()
        self.setFocus()

    @Slot(list)
    def resetProfModifications(self, args):
        self.prof_collections[self.pc_idx].resetModification(*args)
        self.updateProfs()
        self.setFocus()

    def resetProfInterpolation(self):
        self.prof_collections[self.pc_idx].resetInterpolation()
        self.updateProfs()
        self.setFocus()

    def resetVector(self):
        self.prof_collections[self.pc_idx].resetStormMotion()
        self.updateProfs()
        self.setFocus()

    @Slot()
    def toggleCollectObserved(self):
        self.coll_observed = not self.coll_observed
        self.sound.setAllObserved(self.coll_observed)
        self.hodo.setAllObserved(self.coll_observed)

    def loadWidgets(self):
        logging.debug("Calling SPCWindow.loadWidgets")
        ## add the upper-right window insets
        self.grid2.addWidget(self.speed_vs_height, 0, 0, 11, 3)
        self.grid2.addWidget(self.inferred_temp_advection, 0, 3, 11, 2)
        self.grid2.addWidget(self.hodo, 0, 5, 8, 24)
        self.grid2.addWidget(self.storm_slinky, 8, 5, 3, 6)
        self.grid2.addWidget(self.thetae_vs_pressure, 8, 11, 3, 6)
        self.grid2.addWidget(self.srwinds_vs_height, 8, 17, 3, 6)
        self.grid2.addWidget(self.watch_type, 8, 23, 3, 6)

        # Draw the kinematic and convective insets
        self.grid3.addWidget(self.convective, 0, 0)
        self.grid3.addWidget(self.kinematic, 0, 1)

        # Set Left Inset
        self.grid3.addWidget(self.left_inset_ob, 0, 2)

        # Set Right Inset
        self.grid3.addWidget(self.right_inset_ob, 0, 3)

        ## do a check for setting the dendretic growth zone
        if self.left_inset == "WINTER" or self.right_inset == "WINTER":
            self.sound.setDGZ(True)
            self.dgz = True

        ## Do a check for setting the pbl
        if self.left_inset == "FIRE" or self.right_inset == "FIRE":
            self.sound.setPBLLevel(True)
            self.pbl = True

        self.grid.addWidget(self.sound, 0, 0, 3, 1)
        self.grid.addWidget(self.text, 3, 0, 1, 2)

    def advanceTime(self, direction):
        logging.debug("Calling SPCWidget.advanceTime")
        if len(self.prof_collections) == 0 or self.coll_observed:
            return

        prof_col = self.prof_collections[self.pc_idx]
        if prof_col.getMeta('observed'):
            cur_dt = prof_col.getCurrentDate()
            cur_loc = prof_col.getMeta('loc')
            idxs, dts = list(zip(*sorted(((idx, pc.getCurrentDate()) for idx, pc in enumerate(self.prof_collections) if pc.getMeta('loc') == cur_loc and pc.getMeta('observed')), key=lambda x: x[1])))

            dt_idx = dts.index(cur_dt)
            dt_idx = (dt_idx + direction) % len(dts)
            self.pc_idx = idxs[dt_idx]

            cur_dt = self.prof_collections[self.pc_idx].getCurrentDate()
        else:
            cur_dt = prof_col.advanceTime(direction)

        for prof_col in self.prof_collections:
            if not prof_col.getMeta('observed'):
                prof_col.setCurrentDate(cur_dt)

        self.parcel_types = self.convective.pcl_types
        self.updateProfs()

        prof_col = self.prof_collections[self.pc_idx]
        if prof_col.hasMeta('analogfile'):
            match = prof_col.getMeta('analogfile')
            dt = prof_col.getCurrentDate()
            if dt in match:
                self.insets['SARS'].setSelection(match[dt])
            else:
                self.insets['SARS'].clearSelection()
        else:
            self.insets['SARS'].setParent(self.text)
            self.insets['SARS'].clearSelection()

    def advanceHighlight(self, direction):
        self.prof_collections[self.pc_idx].advanceHighlight(direction)
        self.updateProfs()

    def swapProfCollections(self):
        logging.debug("Calling SPCWidget.swapProfCollections")
        # See if we have any other observed profiles loaded at this time.
        prof_col = self.prof_collections[self.pc_idx]
        dt = prof_col.getCurrentDate()
        idxs, pcs = list(zip(*[ (idx, pc) for idx, pc in enumerate(self.prof_collections) if pc.getCurrentDate() == dt or self.coll_observed ]))
        loc_idx = pcs.index(prof_col)
        loc_idx = (loc_idx + 1) % len(pcs)
        self.pc_idx = idxs[loc_idx]

        self.updateProfs()

        if self.prof_collections[self.pc_idx].hasMeta('analogfile'):
            match = self.prof_collections[self.pc_idx].getMeta('analogfile')
            dt = prof_col.getCurrentDate()
            if dt in match:
                self.insets['SARS'].setSelection(match[dt])
            else:
                self.insets['SARS'].clearSelection()
        else:
            self.insets['SARS'].clearSelection()

    def closeEvent(self, e):
        logging.debug("SPCWindow closeEvent:" + str(e))
        self.sound.closeEvent(e)

        for prof_coll in self.prof_collections:
            prof_coll.cancelCopy()

    def makeInsetMenu(self, *exclude):
        # This will make the menu of the available insets.
        self.popupmenu=QMenu("Inset Menu")
        self.menu_ag = QActionGroup(self, exclusive=True)

        for inset in self.available_insets:
            if inset not in exclude:
                inset_action = QAction(self)
                inset_action.setText(SPCWidget.inset_names[inset])
                inset_action.setData(inset)
                inset_action.setCheckable(True)
                inset_action.triggered.connect(self.swapInset)
                a = self.menu_ag.addAction(inset_action)
                self.popupmenu.addAction(a)

    def showCursorMenu(self, pos):
        self.makeInsetMenu(self.left_inset, self.right_inset)
        if self.childAt(pos.x(), pos.y()) is self.right_inset_ob:
            self.inset_to_swap = "RIGHT"
            self.popupmenu.popup(self.mapToGlobal(pos))
            self.setFocus()

        elif self.childAt(pos.x(), pos.y()) is self.left_inset_ob:
            self.inset_to_swap = "LEFT"
            self.popupmenu.popup(self.mapToGlobal(pos))
            self.setFocus()

    def swapInset(self):
        logging.debug("Swapping the " + self.inset_to_swap + " inset.")
        ## This will swap either the left or right inset depending on whether or not the
        ## self.inset_to_swap value is LEFT or RIGHT.
        a = self.menu_ag.checkedAction()

        prefs = dict( (field, value) for (section, field), value in self.config if section == 'preferences')

        if self.inset_to_swap == "LEFT":
            if self.left_inset == "WINTER" and self.dgz:
                self.sound.setDGZ(False)
                self.dgz = False

            if self.left_inset == "FIRE" and self.pbl:
                self.sound.setPBLLevel(False)
                self.pbl = False

            # Delete and re-make the inset.  For some stupid reason, pyside/QT forces you to
            #   delete something you want to remove from the layout.
            self.left_inset_ob.deleteLater()
            self.insets[self.left_inset] = SPCWidget.inset_generators[self.left_inset]()
            self.insets[self.left_inset].setProf(self.default_prof)
            self.insets[self.left_inset].setPreferences(update_gui=False, **prefs)
            self.insets[self.left_inset].setParent(self.text)

            self.left_inset = a.data()
            self.left_inset_ob = self.insets[self.left_inset]
            self.grid3.addWidget(self.left_inset_ob, 0, 2)
            self.config['insets', 'left_inset'] = self.left_inset

        elif self.inset_to_swap == "RIGHT":
            if self.right_inset == "WINTER" and self.dgz:
                self.sound.setDGZ(False)
                self.dgz = False

            if self.right_inset == "FIRE" and self.pbl:
                self.sound.setPBLLevel(False)
                self.pbl = False

            # Delete and re-make the inset.  For some stupid reason, pyside/QT forces you to
            #   delete something you want to remove from the layout.
            self.right_inset_ob.deleteLater()
            self.insets[self.right_inset] = SPCWidget.inset_generators[self.right_inset]()
            self.insets[self.right_inset].setProf(self.default_prof)
            self.insets[self.right_inset].setPreferences(update_gui=False, **prefs)
            self.insets[self.right_inset].setParent(self.text)

            self.right_inset = a.data()
            self.right_inset_ob = self.insets[self.right_inset]
            self.grid3.addWidget(self.right_inset_ob, 0, 3)
            self.config['insets', 'right_inset'] = self.right_inset

        if a.data() == "WINTER":
            self.sound.setDGZ(True)
            self.dgz = True
        if a.data() == "FIRE":
            self.sound.setPBLLevel(True)
            self.pbl = True

        self.setFocus()
        self.update()

class SPCWindow(QMainWindow):
    closed = Signal()

    def __init__(self, **kwargs):
        parent = kwargs.get('parent', None)
        super(SPCWindow, self).__init__(parent=parent)
        logging.debug("Initialize SPCWindow...")
        self.menu_items = []
        self.picker_window = parent
        self.__initUI(**kwargs)

    def __initUI(self, **kwargs):
        kwargs['parent'] = self
        self.spc_widget = SPCWidget(**kwargs)
        self.parent().config_changed.connect(self.spc_widget.updateProfs)
        self.parent().config_changed.connect(self.updateConfig)
        self.setCentralWidget(self.spc_widget)
        self.createMenuBar()

        logging.debug("Determining system platform to resize the window.")
        title = 'SHARPpy: Sounding and Hodograph Analysis and Research Program '
        title += 'in Python'
        self.setWindowTitle(title)

        bg_hex = self.spc_widget.config['preferences', 'bg_color']
        self.setStyleSheet("QMainWindow { background-color: " + bg_hex + "; }")

        ## handle the attribute of the main window
        if platform.system() == 'Windows':
            self.setGeometry(10,30,1180,800)
        else:
            self.setGeometry(0, 0, 1180, 800)

        self.show()
        self.raise_()

    def createMenuBar(self):
        logging.debug("Creating the SPCWindow Menu Bar.")
        bar = self.menuBar()
        filemenu = bar.addMenu("File")

        pref = QAction("Preferences", self)
        filemenu.addAction(pref)
        pref.triggered.connect(self.parent().preferencesbox)

        saveimage = QAction("Save Image", self, shortcut=QKeySequence("Ctrl+S"))
        saveimage.triggered.connect(self.spc_widget.saveimage)
        filemenu.addAction(saveimage)

        savetext = QAction("Save Text", self, shortcut=QKeySequence("Ctrl+Shift+S"))
        savetext.triggered.connect(self.spc_widget.savetext)
        filemenu.addAction(savetext)

        self.profilemenu = bar.addMenu("Profiles")

        self.allobserved = QAction("Collect Observed", self, checkable=True, shortcut=QKeySequence("C"))
        self.allobserved.triggered.connect(self.spc_widget.toggleCollectObserved)
        self.profilemenu.addAction(self.allobserved)

        self.interpolate = QAction("Interpolate Focused Profile", self, shortcut=QKeySequence("I"))
        self.interpolate.triggered.connect(self.interpProf)
        self.profilemenu.addAction(self.interpolate)

        self.resetinterp = QAction("Reset Interpolation", self, shortcut=QKeySequence("I"))
        self.resetinterp.triggered.connect(self.resetProf)
        self.resetinterp.setVisible(False)
        self.profilemenu.addAction(self.resetinterp)

        self.profilemenu.addSeparator()

        self.focus_mapper = QSignalMapper(self)
        self.remove_mapper = QSignalMapper(self)

        self.focus_mapper.mapped[str].connect(self.spc_widget.setProfileCollection)
        self.remove_mapper.mapped[str].connect(self.rmProfileCollection)

    def createProfileMenu(self, prof_col):
        logging.debug("Creating the SPCWindow Profile Menu.")
        menu_name = self.createMenuName(prof_col)
        prof_menu = self.profilemenu.addMenu(menu_name)

        focus = QAction("Focus", self)
        focus.triggered.connect(self.focus_mapper.map)
        self.focus_mapper.setMapping(focus, menu_name)
        prof_menu.addAction(focus)

        remove = QAction("Remove", self)
        remove.triggered.connect(self.remove_mapper.map)
        self.remove_mapper.setMapping(remove, menu_name)
        prof_menu.addAction(remove)

        if len(self.menu_items) == 0:
            remove.setVisible(False)

        self.menu_items.append(prof_menu)

    def removeProfileMenu(self, menu_name):
        menu_items = [ mitem for mitem in self.menu_items if mitem.title() == menu_name ]
        for mitem in menu_items:
            mitem.menuAction().setVisible(False)

    def addProfileCollection(self, prof_col, focus=True, check_integrity=True):
        logging.debug("Calling SPCWindow.addProfileCollection")

        if check_integrity is True:
            logging.debug("Testing data integrity")
            exit_code = self.testDataIntegrity(prof_col.getHighlightedProf())
            if exit_code == 0:
                if len(self.menu_items) == 0:
                    self.focusPicker()
                    self.close()
                else:
                    return

        menu_name = self.createMenuName(prof_col)
        if any( mitem.title() == menu_name and mitem.menuAction().isVisible() for mitem in self.menu_items ):
            self.spc_widget.setProfileCollection(menu_name)
            return

        # JTS - keep "collect observed" option active, regardless of profile type.
        # if not prof_col.getMeta('observed'):
            # self.allobserved.setDisabled(True)
            # self.allobserved.setChecked(False)

        self.createProfileMenu(prof_col)

        visible_mitems = [ mitem for mitem in self.menu_items if mitem.menuAction().isVisible() ]
        if len(visible_mitems) > 1:
            actions = visible_mitems[0].actions()
            names = [ act.text() for act in actions ]
            actions[names.index("Remove")].setVisible(True)

        try:
            self.spc_widget.addProfileCollection(prof_col, menu_name, focus=focus)
        except Exception as exc:
            logging.exception(exc)
            print("OOPS:", exc)
            ### TODO: This may be a good place to output a copy of the offending data (useful for debugging observed data).
            if len(self.menu_items) == 1:
                self.focusPicker()
                self.close()
            else:
                self.rmProfileCollection(menu_name)
            raise

    def testDataIntegrity(self, prof):
        try:
            prof.checkDataIntegrity()
        except Exception as e:
            logging.exception(e)
            msgBox = QMessageBox()
            msgBox.setText("SHARPpy has detected that the data you are attempting to load may have errors.")
            msgBox.setInformativeText("Do you want to still try and load the data?")
            msgBox.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
            msgBox.setDefaultButton(QMessageBox.No)
            msgBox.setDetailedText(str(e).split(',')[1].replace(')', ''))
            ret = msgBox.exec_()
            if ret == QMessageBox.Yes:
                return 1
            else:
                return 0
        return 1


    @Slot(str)
    def rmProfileCollection(self, menu_name):
        logging.debug("Calling SPCWindow.rmProfileCollection")
        self.removeProfileMenu(menu_name)
        self.spc_widget.rmProfileCollection(menu_name)

        if self.spc_widget.isAllObserved():
            self.allobserved.setDisabled(False)

        visible_mitems = [ mitem for mitem in self.menu_items if mitem.menuAction().isVisible() ]
        if len(visible_mitems) == 1:
            actions = visible_mitems[0].actions()
            names = [ act.text() for act in actions ]
            actions[names.index("Remove")].setVisible(False)

    def keyPressEvent(self, e):
        if e.key() == Qt.Key_Left: # Step forward/backwards in time
            self.spc_widget.advanceTime(-1)
            self.setInterpolated(self.spc_widget.isInterpolated())
        elif e.key() == Qt.Key_Right:
            self.spc_widget.advanceTime(1)
            self.setInterpolated(self.spc_widget.isInterpolated())
        elif e.key() == Qt.Key_Up: # Change focus from member to member of the ensemble
            self.spc_widget.advanceHighlight(1)
        elif e.key() == Qt.Key_Down:
            self.spc_widget.advanceHighlight(-1)
        elif e.key() == Qt.Key_Space:
            # Swap the profile collections
            self.spc_widget.swapProfCollections()
            self.setInterpolated(self.spc_widget.isInterpolated())
        elif e.matches(QKeySequence.Save):
            # Save an image
            self.spc_widget.saveimage()
        elif e.key() == Qt.Key_W:
            # Return focus to the Sounding Picker
            self.focusPicker()

    def closeEvent(self, e):
        logging.debug("Calling SPCWindow.closeEvent")

        self.spc_widget.closeEvent(e)
        self.closed.emit()

    def closeIfEmpty(self):
        logging.debug("Calling SPCWindow.closeIfEmpty")
        visible_mitems = [ mitem for mitem in self.menu_items if mitem.menuAction().isVisible() ]
        if len(visible_mitems) < 1:
            self.close()

    def createMenuName(self, prof_col):
        pc_loc = prof_col.getMeta('loc')
        pc_date = prof_col.getMeta('run').strftime("%d/%H%MZ")
        pc_model = prof_col.getMeta('model')

        # JTS - For NUCAPS case study data sources,
        # construct pc_date string from pc_loc since obs times list is empty.
        if pc_model == "NUCAPS Case Study NOAA-20" \
            or pc_model == "NUCAPS Case Study Suomi-NPP" \
            or pc_model == "NUCAPS Case Study Aqua" \
            or pc_model == "NUCAPS Case Study MetOp-A" \
            or pc_model == "NUCAPS Case Study MetOp-B" \
            or pc_model == "NUCAPS Case Study MetOp-C":
            date = pc_loc.split('_')[0]
            time = pc_loc.split('_')[1]
            day = date[4:6]
            hour = time[0:2]
            minute = time[2:4]
            pc_date = f'{day}/{hour}{minute}Z'

        return "%s (%s %s)" % (pc_loc, pc_date, pc_model)

    def interpProf(self):
        self.setInterpolated(True)
        self.spc_widget.interpProf()

    def resetProf(self):
        self.setInterpolated(False)
        self.spc_widget.resetProfInterpolation()

    def setInterpolated(self, is_interpolated):
        self.resetinterp.setVisible(is_interpolated)
        self.interpolate.setVisible(not is_interpolated)

    @Slot()
    def updateConfig(self):
        bg_hex = self.spc_widget.config['preferences', 'bg_color']
        sheet = self.styleSheet()
        sheet = _modifySheet(sheet, 'background-color', bg_hex)
        self.setStyleSheet(sheet)
        self.update()

    def focusPicker(self):
        logging.debug("Calling SPCWindow.focusPicker")
        if self.picker_window is not None:
            self.picker_window.activateWindow()
            self.picker_window.setFocus()
            self.picker_window.raise_()

if __name__ == '__main__':
    app_frame = QApplication([])
    tester = SPCWindow()
    tester.show()
    app_frame.exec_()



''' Interpolation Routines '''
from __future__ import division
import numpy as np
import numpy.ma as ma
import numpy.testing as npt
from . import utils, thermo
from . constants import *


__all__ = ['pres', 'hght', 'temp', 'dwpt', 'vtmp', 'components', 'vec']
__all__ += ['thetae', 'wetbulb', 'theta', 'mixratio']
__all__ += ['to_agl', 'to_msl']


def pres(prof, h):
    '''
    Interpolates the given data to calculate a pressure at a given height

    Parameters
    ----------
    prof : profile object
        Profile object
    h : number, numpy array
        Height (m) of the level for which pressure is desired

    Returns
    -------
    Pressure (hPa) at the given height : number, numpy array

    '''
    return generic_interp_hght(h, prof.hght, prof.logp, log=True)


def hght(prof, p):
    '''
    Interpolates the given data to calculate a height at a given pressure

    Parameters
    ----------
    prof : profile object
        Profile object
    p : number, numpy array
        Pressure (hPa) of the level for which height is desired

    Returns
    -------
    Height (m) at the given pressure : number, numpy array

    '''
    # Note: numpy's interpolation routine expects the interpolation
    # routine to be in ascending order. Because pressure decreases in the
    # vertical, we must reverse the order of the two arrays to satisfy
    # this requirement.
    return generic_interp_pres(ma.log10(p), prof.logp[::-1], prof.hght[::-1])

def omeg(prof, p):
    '''
    Interpolates the given data to calculate a omega at a given pressure

    Parameters
    ----------
    prof : profile object
        Profile object
    p : number, numpy array
        Pressure (hPa) of the level for which temperature is desired

    Returns
    -------
    Omega (microbars/second) at the given pressure : number, numpy array

    '''
    # Note: numpy's interpolation routine expects the interpolation
    # routine to be in ascending order. Because pressure decreases in the
    # vertical, we must reverse the order of the two arrays to satisfy
    # this requirement.
    return generic_interp_pres(ma.log10(p), prof.logp[::-1], prof.omeg[::-1])

def temp(prof, p):
    '''
    Interpolates the given data to calculate a temperature at a given pressure

    Parameters
    ----------
    prof : profile object
        Profile object
    p : number, numpy array
        Pressure (hPa) of the level for which temperature is desired

    Returns
    -------
    Temperature (C) at the given pressure : number, numpy array

    '''
    # Note: numpy's interpolation routine expects the interpolation
    # routine to be in ascending order. Because pressure decreases in the
    # vertical, we must reverse the order of the two arrays to satisfy
    # this requirement.
    return generic_interp_pres(ma.log10(p), prof.logp[::-1], prof.tmpc[::-1])

def thetae(prof, p):
    '''
        Interpolates the given data to calculate theta-e at a given pressure

        Parameters
        ----------
        prof : profile object
        Profile object
        p : number, numpy array
        Pressure (hPa) of the level for which temperature is desired

        Returns
        -------
        Theta-E (C) at the given pressure : number, numpy array

        '''
    # Note: numpy's interpolation routine expects the interpolation
    # routine to be in ascending order. Because pressure decreases in the
    # vertical, we must reverse the order of the two arrays to satisfy
    # this requirement.
    return generic_interp_pres(ma.log10(p), prof.logp[::-1], prof.thetae[::-1])

def mixratio(prof, p):
    '''
        Interpolates the given data to calculate water vapor mixing ratio at a given pressure

        Parameters
        ----------
        prof : profile object
        Profile object
        p : number, numpy array
        Pressure (hPa) of the level for which mixing ratio is desired

        Returns
        -------
        Water vapor mixing ratio (g/kg) at the given pressure : number, numpy array

        '''
    # Note: numpy's interpolation routine expects the interpolation
    # routine to be in ascending order. Because pressure decreases in the
    # vertical, we must reverse the order of the two arrays to satisfy
    # this requirement.
    return generic_interp_pres(ma.log10(p), prof.logp[::-1], prof.wvmr[::-1])


def theta(prof, p):
    '''
        Interpolates the given data to calculate theta at a given pressure

        Parameters
        ----------
        prof : profile object
        Profile object
        p : number, numpy array
        Pressure (hPa) of the level for which potential temperature is desired

        Returns
        -------
        Theta (C) at the given pressure : number, numpy array

        '''
    # Note: numpy's interpolation routine expects the interpolation
    # routine to be in ascending order. Because pressure decreases in the
    # vertical, we must reverse the order of the two arrays to satisfy
    # this requirement.
    return generic_interp_pres(ma.log10(p), prof.logp[::-1], prof.theta[::-1])

def wetbulb(prof, p):
    '''
        Interpolates the given data to calculate a wetbulb temperature at a given pressure

        Parameters
        ----------
        prof : profile object
        Profile object
        p : number, numpy array
        Pressure (hPa) of the level for which wetbulb temperature is desired

        Returns
        -------
        Wetbulb temperature (C) at the given pressure : number, numpy array

        '''
    # Note: numpy's interpolation routine expects the interpolation
    # routine to be in ascending order. Because pressure decreases in the
    # vertical, we must reverse the order of the two arrays to satisfy
    # this requirement.
    return generic_interp_pres(ma.log10(p), prof.logp[::-1], prof.wetbulb[::-1])

def dwpt(prof, p):
    '''
    Interpolates the given data to calculate a dew point temperature
    at a given pressure

    Parameters
    ----------
    prof : profile object
        Profile object
    p : number, numpy array
        Pressure (hPa) of the level for which dew point temperature is desired

    Returns
    -------
    Dew point tmperature (C) at the given pressure : number, numpy array

    '''
    # Note: numpy's interpolation routine expects the interpolation
    # routine to be in ascending order. Because pressure decreases in the
    # vertical, we must reverse the order of the two arrays to satisfy
    # this requirement.
    return generic_interp_pres(ma.log10(p), prof.logp[::-1], prof.dwpc[::-1])


def vtmp(prof, p):
    '''
    Interpolates the given data to calculate a virtual temperature
    at a given pressure

    Parameters
    ----------
    prof : profile object
        Profile object
    p : number, numpy array
        Pressure (hPa) of the level for which virtual temperature is desired

    Returns
    -------
    Virtual tmperature (C) at the given pressure : number, numpy array

    '''
    return generic_interp_pres(ma.log10(p), prof.logp[::-1], prof.vtmp[::-1])


def components(prof, p):
    '''
    Interpolates the given data to calculate the U and V components at a
    given pressure

    Parameters
    ----------
    prof : profile object
        Profile object
    p : number, numpy array
        Pressure (hPa) of a level

    Returns
    -------
    U and V components at the given pressure (kts) : number, numpy array
    '''
    # Note: numpy's interpolation routine expects the interpolation
    # routine to be in ascending order. Because pressure decreases in the
    # vertical, we must reverse the order of the two arrays to satisfy
    # this requirement.
    if prof.wdir.count() == 0:
        # JTS - Fixed a bug where clicking "Interpolate Focused Profile" throws an error for NUCAPS.
        return ma.masked_where(ma.ones(np.shape(p)), p), ma.masked_where(ma.ones(np.shape(p)), p)
    U = generic_interp_pres(ma.log10(p), prof.logp[::-1], prof.u[::-1])
    V = generic_interp_pres(ma.log10(p), prof.logp[::-1], prof.v[::-1])
    return U, V


def vec(prof, p):
    '''
    Interpolates the given data to calculate the wind direction and speed
    at a given pressure

    Parameters
    ----------
    p : number, numpy array
        Pressure (hPa) of a level
    prof : profile object
        Profile object

    Returns
    -------
    Wind direction (degrees) and magnitude (kts) at the given pressure : number, numpy array
    '''
    U, V = components(prof, p)
    return utils.comp2vec(U, V)


def to_agl(prof, h):
    '''
    Convert a height from mean sea-level (MSL) to above ground-level (AGL)

    Parameters
    ----------
    h : number, numpy array
        Height of a level
    prof : profile object
        Profile object

    Returns
    -------
    Converted height (m AGL) : number, numpy array

    '''
    return h - prof.hght[prof.sfc]


def to_msl(prof, h):
    '''
    Convert a height from above ground-level (AGL) to mean sea-level (MSL)

    Parameters
    ----------
    h : number, numpy array
        Height of a level
    prof : profile object
        Profile object

    Returns
    -------
    Converted height (m MSL) : number, numpy array

    '''
    return h + prof.hght[prof.sfc]


def generic_interp_hght(h, hght, field, log=False):
    '''
    Generic interpolation routine

    Parameters
    ----------
    h : number, numpy array
        Height (m) of the level for which pressure is desired
    hght : numpy array
        The array of heights
    field : numpy array
        The variable which is being interpolated
    log : bool
        Flag to determine whether the 'field' variable is in log10 space

    Returns
    -------
    Value of the 'field' variable at the given height : number, numpy array

    '''
    if field.count() == 0 or hght.count() == 0:
        return ma.masked_where(ma.ones(np.shape(h)), h) # JTS

    if ma.isMaskedArray(hght):
        # Multiplying by ones ensures that the result is an array, not a single value ... which
        # happens sometimes ... >.<
        not_masked1 = ~hght.mask * np.ones(hght.shape, dtype=bool)
    else:
        not_masked1 = np.ones(hght.shape)

    if ma.isMaskedArray(field):
        not_masked2 = ~field.mask * np.ones(field.shape, dtype=bool)
    else:
        not_masked2 = np.ones(field.shape)

    not_masked = not_masked1 * not_masked2

    field_intrp = np.interp(h, hght[not_masked], field[not_masked], left=ma.masked, right=ma.masked)

    if hasattr(h, 'shape') and h.shape == tuple():
        h = h[()]

    # Another bug fix: np.interp() returns masked values as nan. We want ma.masked, dangit!
    field_intrp = ma.where(np.isnan(field_intrp), ma.masked, field_intrp)

    # ma.where() returns a 0-d array when the arguments are floats, which confuses subsequent code.
    if hasattr(field_intrp, 'shape') and field_intrp.shape == tuple():
        field_intrp = field_intrp[()]

    if log:
        return 10 ** field_intrp
    else:
        return field_intrp

def generic_interp_pres(p, pres, field):
    '''
    Generic interpolation routine

    Parameters
    ----------
    p : number, numpy array
        Pressure (hPa) of the level for which the field variable is desired
    pres : numpy array
        The array of pressure
    field : numpy array
        The variable which is being interpolated
    log : bool
        Flag to determine whether the 'field' variable is in log10 space

    Returns
    -------
    Value of the 'field' variable at the given pressure : number, numpy array

    '''
    if field.count() == 0 or pres.count() == 0:
        return ma.masked_where(ma.ones(np.shape(p)), p) # JTS

    if ma.isMaskedArray(pres):
        not_masked1 = ~pres.mask * np.ones(pres.shape, dtype=bool)
    else:
        not_masked1 = np.ones(pres.shape, dtype=bool)
        not_masked1[:] = True

    if ma.isMaskedArray(field):
        not_masked2 = ~field.mask * np.ones(field.shape, dtype=bool)
    else:
        not_masked2 = np.ones(field.shape, dtype=bool)
        not_masked2[:] = True

    not_masked = not_masked1 * not_masked2

    field_intrp = np.interp(p, pres[not_masked], field[not_masked], left=ma.masked, right=ma.masked)

    if hasattr(p, 'shape') and p.shape == tuple():
        p = p[()]

    # Another bug fix: np.interp() returns masked values as nan. We want ma.masked, dangit!
    field_intrp = ma.where(np.isnan(field_intrp), ma.masked, field_intrp)

    # ma.where() returns a 0-d array when the arguments are floats, which confuses subsequent code.
    if hasattr(field_intrp, 'shape') and field_intrp.shape == tuple():
        field_intrp = field_intrp[()]

    return field_intrp




from qtpy.QtCore import *
from qtpy.QtGui import *
from qtpy.QtWidgets import *

from collections import OrderedDict
import numpy as np

import os
import sys

def resource_path():
    if hasattr(sys, '_MEIPASS'):
        return os.path.join(sys._MEIPASS, "rc")
    return os.path.join(os.path.dirname(__file__), "..", "..", "rc")

class ColorSwatch(QWidget):
    """
    A color swatch widget for displaying and selecting colors.
    """
    def __init__(self, color, parent=None):
        """
        Construct a ColorSwatch class
        color:  A QColor specifying the starting color for the swatch.
        """
        super(ColorSwatch, self).__init__(parent=parent)
        self._color = color

    def setColor(self, new_color):
        """
        Set the color of the swatch.
        new_color:  A QColor containing the new color.
        """
        self._color = new_color

    def getColor(self):
        """
        Returns the current color as a QColor
        """
        return self._color

    def getHexColor(self):
        """
        Returns the current color as a hex string.
        """
        red = "%02x" % self.getColor().red()
        green = "%02x" % self.getColor().green()
        blue = "%02x" % self.getColor().blue()
        return "#%s%s%s" % (red, green, blue)

    def paintEvent(self, e):
        """
        Paint event handler
        """
        qp = QPainter()
        qp.begin(self)
        qp.setBrush(QBrush(self._color))
        qp.drawRect(0, 0, self.width(), self.height())
        qp.end()

    def mousePressEvent(self, e):
        """
        Mouse press event handler (opens a dialog to select a new color for the swatch).
        """
        color_choice = QColorDialog.getColor(self.getColor())
        if color_choice.isValid():
            self.setColor(color_choice)

    def enterEvent(self, e):
        """
        Enter event handler (sets the cursor to a pointing hand).
        """
        self.setCursor(Qt.PointingHandCursor)

    def leaveEvent(self, e):
        """
        Leave event handler (sets the cursor to whatever it was before).
        """
        self.unsetCursor()


class ColorPreview(QWidget):

    _color_imgs = {
        'standard': 'sample_std.png',
        'inverted': 'sample_inv.png',
        'protanopia': 'sample_pro.png',
    }

    def __init__(self, styles, default='standard', **kwargs):
        super(ColorPreview, self).__init__(**kwargs)
        self._img_path =  resource_path() #os.path.join(os.path.dirname(__file__), "..", "..", "rc")
        self._styles = [ s.lower() for s in styles ]
        self.changeImage(self._styles.index(default))

        self._base_x, self._base_y = 118, 80
        self._aspect = float(self._base_x) / self._base_y

        self.setMinimumSize(self._base_x, self._base_y)
        self.show()

    @Slot(str)
    def changeImage(self, img_index):
        self._current = self._styles[img_index]
        self._img = QPixmap(os.path.join(self._img_path, ColorPreview._color_imgs[self._current]))
        self.update()

    def paintEvent(self, e):
        qp = QPainter()
        qp.begin(self)
        qp.setBrush(QBrush(QColor("#000000")))
        qp.drawPixmap(0, 0, self.width(), self.height(), self._img, 0, 0, self._img.width(), self._img.height())
        qp.end()


class PrefDialog(QDialog):
    """
    The preferences dialog box for SHARPpy.
    """

    _styles = {
        'standard': {
            'bg_color': '#000000',
            'fg_color': '#ffffff',

            'temp_color': '#ff0000',
            'dewp_color': '#00ff00',
            'wetb_color': '#00ffff',
            'skew_dgz_color':'#f5d800',
            'skew_itherm_color':'#555555',
            'skew_itherm_hgz_color':'#0000ff',
            'skew_adiab_color':'#333333',
            'skew_mixr_color':'#006600',
            'skew_hgt_color':'#ff0000',
            'skew_lcl_mkr_color':'#00ff00',
            'skew_lfc_mkr_color':'#ffff00',
            'skew_el_mkr_color':'#ff00ff',

            'hodo_itach_color':'#555555',
            'hodo_crit_color':'#00ffff',

            'eff_inflow_color':'#04dbd8',

            '0_3_color': '#ff0000',
            '3_6_color': '#00ff00',
            '6_9_color': '#ffff00',
            '9_12_color': '#00ffff',
            '12_15_color': '#00ffff',

            'pwat_m3_color':'#ff7f00',
            'pwat_m2_color':'#ee9a00',
            'pwat_m1_color':'#ffdab9',
            'pwat_p1_color':'#98fb98',
            'pwat_p2_color':'#66cd00',
            'pwat_p3_color':'#00ff00',

            'alert_l1_color':'#775000',
            'alert_l2_color':'#996600',
            'alert_l3_color':'#ffffff',
            'alert_l4_color':'#ffff00',
            'alert_l5_color':'#ff0000',
            'alert_l6_color':'#e700df',
            'alert_lscp_color':'#00ffff',

            'stp_box_color':'#00ff00',
            'stp_line_color':'#0080ff',

            'pcl_sel_color':'#00ffff',
            'pcl_cin_hi_color':'#993333',
            'pcl_cin_md_color':'#996600',
            'pcl_cin_lo_color':'#00ff00',

            'spd_itach_color':'#9d5736',

            'srw_clsc_color':'#b1019a',
            'srw_trace_color':'#ff0000',
            'srw_0_2_color':'#8b0000',
            'srw_4_6_color':'#6495ed',
            'srw_9_11_color':'#9400d3',

            'winter_dgz_color':'#ffff00',

            'watch_pdstor_color':'#ff00ff',
            'watch_tor_color':'#ff0000',
            'watch_svr_color':'#ffff00',
            'watch_mrglsvr_color':'#0099cc',
            'watch_flood_color':'#5ffb17',
            'watch_blizzard_color':'#3366ff',
            'watch_fire_color':'#ff9900',
            'watch_heat_color':'#c85a17',
            'watch_none_color':'#ffcc33',
        },
        'inverted': {
            'bg_color': '#ffffff',
            'fg_color': '#000000',

            'temp_color': '#dd0000',
            'dewp_color': '#00dd00',
            'wetb_color': '#00cccc',
            'skew_dgz_color':'#f5d800',
            'skew_itherm_color':'#bbbbbb',
            'skew_itherm_hgz_color':'#6666ff',
            'skew_adiab_color':'#dddddd',
            'skew_mixr_color':'#adebad',
            'skew_hgt_color':'#dd0000',
            'skew_lcl_mkr_color':'#00bb00',
            'skew_lfc_mkr_color':'#bbbb00',
            'skew_el_mkr_color':'#bb00bb',

            'hodo_itach_color':'#bbbbbb',
            'hodo_crit_color':'#00cccc',

            'eff_inflow_color': '#00cccc',

            '0_3_color': '#dd0000',
            '3_6_color': '#00dd00',
            '6_9_color': '#dddd00',
            '9_12_color': '#00dddd',
            '12_15_color': '#00dddd',

            'pwat_m3_color':'#ff9900',
            'pwat_m2_color':'#cc7a00',
            'pwat_m1_color':'#804d00',
            'pwat_p1_color':'#006600',
            'pwat_p2_color':'#00b300',
            'pwat_p3_color':'#00ff00',

            'alert_l1_color':'#dfbb9f',
            'alert_l2_color':'#cc9666',
            'alert_l3_color':'#666666',
            'alert_l4_color':'#999900',
            'alert_l5_color':'#cc0000',
            'alert_l6_color':'#e700df',
            'alert_lscp_color':'#00cccc',

            'stp_box_color':'#00aa00',
            'stp_line_color':'#66b3ff',

            'pcl_sel_color':'#00cccc',
            'pcl_cin_hi_color':'#dfbb9f',
            'pcl_cin_md_color':'#cc9666',
            'pcl_cin_lo_color':'#00cc00',

            'spd_itach_color':'#ebd2c6',

            'srw_clsc_color':'#fe81ed',
            'srw_trace_color':'#dd0000',
            'srw_0_2_color':'#8b0000',
            'srw_4_6_color':'#6495ed',
            'srw_9_11_color':'#9400d3',

            'winter_dgz_color':'#999900',

            'watch_pdstor_color':'#dd00dd',
            'watch_tor_color':'#dd0000',
            'watch_svr_color':'#aaaa00',
            'watch_mrglsvr_color':'#0099cc',
            'watch_flood_color':'#5ffb17',
            'watch_blizzard_color':'#3366ff',
            'watch_fire_color':'#ff9900',
            'watch_heat_color':'#c85a17',
            'watch_none_color':'#cc9900',
        },
        'protanopia': {
            'bg_color': '#000000',
            'fg_color': '#ffffff',

            'temp_color': '#ff0000',
            'dewp_color': '#00ffff',
            'wetb_color': '#6666ff',
            'skew_dgz_color':'#f5d800',
            'skew_itherm_color':'#555555',
            'skew_itherm_hgz_color':'#0000ff',
            'skew_adiab_color':'#333333',
            'skew_mixr_color':'#006600',
            'skew_hgt_color':'#ff0000',
            'skew_lcl_mkr_color':'#00ff00',
            'skew_lfc_mkr_color':'#ffff00',

            'hodo_itach_color':'#555555',
            'hodo_crit_color':'#00ffff',

            'eff_inflow_color': '#00ffff',

            '0_3_color': '#ff0000',
            '3_6_color': '#0000ff',
            '6_9_color': '#ff9900',
            '9_12_color': '#00ffff',
            '12_15_color': '#00ffff',

            'pwat_m3_color':'#ff7f00',
            'pwat_m2_color':'#ee9a00',
            'pwat_m1_color':'#ffdab9',
            'pwat_p1_color':'#98fb98',
            'pwat_p2_color':'#66cd00',
            'pwat_p3_color':'#00ff00',

            'alert_l1_color':'#775000',
            'alert_l2_color':'#996600',
            'alert_l3_color':'#00ff99',
            'alert_l4_color':'#ffffff',
            'alert_l5_color':'#00ffff',
            'alert_l6_color':'#ff00ff',
            'alert_lscp_color':'#ffff00',

            'stp_box_color':'#00ff00',
            'stp_line_color':'#0080ff',

            'pcl_sel_color':'#00ffff',
            'pcl_cin_hi_color':'#993333',
            'pcl_cin_md_color':'#996600',
            'pcl_cin_lo_color':'#00ff00',

            'spd_itach_color':'#9d5736',

            'srw_clsc_color':'#b1019a',
            'srw_trace_color':'#ff0000',
            'srw_0_2_color':'#8b0000',
            'srw_4_6_color':'#6495ed',
            'srw_9_11_color':'#9400d3',

            'winter_dgz_color':'#ffff00',

            'watch_pdstor_color':'#ff00ff',
            'watch_tor_color':'#ff0000',
            'watch_svr_color':'#ffff00',
            'watch_mrglsvr_color':'#0099cc',
            'watch_flood_color':'#5ffb17',
            'watch_blizzard_color':'#3366ff',
            'watch_fire_color':'#ff9900',
            'watch_heat_color':'#c85a17',
            'watch_none_color':'#ffcc33',
        }
    }

    def __init__(self, config, parent=None):
        """
        Construct the preferences dialog box.
        config: A Config object containing the user's configuration.
        """
        super(PrefDialog, self).__init__(parent=parent)
        self._config = config

        self.__initUI()

    def __initUI(self):
        """
        Set up the user interface [private method].
        """
        self.setWindowTitle("SHARPpy Preferences")
        main_layout = QVBoxLayout()
        button_layout = QHBoxLayout()

        self.layout = QGridLayout()
        self.setLayout(main_layout)

        self.accept_button = QPushButton("Accept")
        self.accept_button.setDefault(True)
        self.accept_button.clicked.connect(self.applyChanges)
        self.cancel_button = QPushButton("Cancel")
        self.cancel_button.clicked.connect(self.rejectChanges)

        button_layout.addWidget(self.accept_button)
        button_layout.addWidget(self.cancel_button)

        main_layout.addLayout(self.layout)
        main_layout.addLayout(button_layout)

        tab_widget = QTabWidget()

        color_widget = self._createColorWidget()
        tab_widget.addTab(color_widget, "Colors")

        map_widget = self._createMapWidget()
#       tab_widget.addTab(map_widget, "Map")

        misc_widget = self._createMiscWidget()
        readout_widget = self._createReadoutWidget()
        tab_widget.addTab(misc_widget, "Units")
        tab_widget.addTab(readout_widget, "Readout")

        self.layout.addWidget(tab_widget, 0, 0, 1, 1)

    def _createReadoutWidget(self):
        layout = QVBoxLayout()
        box = QWidget()
        label = QLabel("Top Right Readout Variable:")
        box.setLayout(layout)
        layout.addWidget(label)
        self.combo1 = QComboBox()
        layout.addWidget(self.combo1)
        self.combo2 = QComboBox()
        layout.addWidget(QLabel("Bottom Right Readout Variable:"))
        layout.addWidget(self.combo2)
        self.variables = {"Temperature (C)": 'tmpc', 'Dewpoint (C)': 'dwpc',\
                          "Equiv. Potential Temp. (K)": 'thetae',\
                          "Wetbulb Temperature (C)": 'wetbulb',\
                          "Potential Temperature (K)": 'theta',\
                          "Water Vapor Mixing Ratio (g/kg)": 'wvmr',\
                          "Vertical Velocity (mb/hr)": 'omeg'}
        for k in self.variables.keys():
            self.combo1.addItem(k)
            self.combo2.addItem(k)

        idx1 = np.where(np.asarray(list(self.variables.values())) == self._config['preferences', 'readout_tr'])[0][0]
        idx2 = np.where(np.asarray(list(self.variables.values())) == self._config['preferences', 'readout_br'])[0][0]
        self.combo1.setCurrentIndex(idx1)
        self.combo2.setCurrentIndex(idx2)

        return box

    def _createColorWidget(self):
        colors_box = QWidget()
        colors_layout = QVBoxLayout()
        colors_box.setLayout(colors_layout)

        color_styles = ['Standard', 'Inverted', 'Protanopia']
        self._color_style = self._config['preferences', 'color_style']

        def updateStyle(style_idx):
            self._color_style = color_styles[style_idx].lower()

        colors_list = QComboBox()
        colors_list.addItems(color_styles)
        colors_list.setCurrentIndex(color_styles.index(self._color_style.title()))
        colors_layout.addWidget(colors_list)

        colors_prvw = ColorPreview(color_styles, default=self._color_style, parent=self)
        colors_layout.addWidget(colors_prvw)
        colors_list.activated.connect(colors_prvw.changeImage)
        colors_list.activated.connect(updateStyle)
        return colors_box

    def _createMapWidget(self):
        return QWidget()

    def _createMiscWidget(self):
        misc_box = QWidget()
        layout = QVBoxLayout()
        misc_box.setLayout(layout)

        temp_units_box, self.temp_units = PrefDialog._createRadioSet("Surface Temperature Units", ["Fahrenheit", "Celsius"], default=self._config['preferences', 'temp_units'])
        layout.addWidget(temp_units_box)

        wind_units_box, self.wind_units = PrefDialog._createRadioSet("Wind Units", ["knots", "m/s"], default=self._config['preferences', 'wind_units'])
        layout.addWidget(wind_units_box)

        pw_units_box, self.pw_units = PrefDialog._createRadioSet("Precipitable Water Vapor Units", ["in", "cm"], default=self._config['preferences', 'pw_units'])
        layout.addWidget(pw_units_box)

        return misc_box

    @staticmethod
    def _createColorBox(name, default_color):
        """
        Create a color swatch and label [private static method]
        name:   The label on the color swatch.
        default_color:  The starting color for the swatch as a hex string.

        Returns a QWidget and a ColorSwatch object.
        """
        swatch = ColorSwatch(QColor(default_color))
        swatch.setMaximumWidth(40)
        swatch.setMinimumWidth(40)

        label = QLabel(name)

        colorbox = QWidget()
        layout = QHBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        colorbox.setLayout(layout)

        layout.addWidget(swatch)
        layout.addWidget(label)
        return colorbox, swatch

    @staticmethod
    def _createRadioSet(set_name, opt_names, default=None, orientation='horizontal'):
        """
        Create a set of radio buttons [private static method]
        set_name:    Name of the group of buttons
        opt_names:   A list of names for the radio buttons.
        default:     The name of the button to be selected initially.
        orientation: The direction to arrange the radio buttons. Accepted values are 'horizontal'
                        and 'vertical'. Default is 'horizontal'.
        Returns a QGroupBox and a dictionary with names as keys of QRadioButton's as values.
        """
        radios = {}
        radio_box = QGroupBox(set_name)

        if orientation == 'horizontal':
            radio_layout = QHBoxLayout()
        elif orientation == 'vertical':
            radio_layout = QVBoxLayout()
        radio_box.setLayout(radio_layout)

        for oname in opt_names:
            radios[oname] = QRadioButton(oname)
            if oname == default:
                radios[oname].setChecked(True)

            radio_layout.addWidget(radios[oname])

        return radio_box, radios

    def _applyRadio(self, config_name, radio):
        """
        Apply the radio button selection to the configuration [private method].
        config_name:    Field in the configuration file to set.
        radio:          Dictionary with names of the radio buttons as keys and QRadioButtons as values.
        """
        for radio_name, rad in radio.items():
            if rad.isChecked():
                self._config['preferences', config_name] = radio_name

    def applyChanges(self):
        """
        Apply the state of the preferences box to the configuration and close the box.
        """
        self._applyRadio('temp_units', self.temp_units)
        self._applyRadio('wind_units', self.wind_units)
        self._applyRadio('pw_units', self.pw_units)

        self._config['preferences', 'color_style'] = self._color_style
        for item, color in PrefDialog._styles[self._color_style].items():
            self._config['preferences', item] = color

        self._config['preferences', 'readout_tr'] = self.variables[self.combo1.currentText()]
        self._config['preferences', 'readout_br'] = self.variables[self.combo2.currentText()]

        self.accept()

    def rejectChanges(self):
        """
        Close the box without applying the changes to the configuration.
        """
        self.reject()

    @staticmethod
    def initConfig(config):
        """
        Initialize the configuration with the user preferences [static method].
        config: A Config object containing the configuration.
        """
        pref_config = {
            ('preferences', 'temp_units'):'Fahrenheit',
            ('preferences', 'wind_units'):'knots',
            ('preferences', 'pw_units'):'in',
            ('preferences', 'color_style'):'standard',
            ('preferences', 'readout_br'): 'dwpc',
            ('preferences', 'readout_tr'): 'tmpc'
        }

        color_config = dict((('preferences', k), v) for k, v in PrefDialog._styles['standard'].items())
        pref_config.update(color_config)

        config.initialize(pref_config)

        color_style = config['preferences', 'color_style']
        for item, color in PrefDialog._styles[color_style].items():
            config['preferences', item] = color



''' Create the Sounding (Profile) Object '''
from __future__ import division
import numpy as np
import numpy.ma as ma
import getpass
from datetime import datetime
from . import utils, winds, params, interp, thermo, watch_type, fire
from . qc_tools import *
from . sars import hail, supercell
from . pwv import pwv_climo
from . constants import MISSING
import logging
import warnings

def create_profile(**kwargs):
    '''
    This is a wrapper function for constructing Profile objects
    and objects that inherit from the Profile class. This will
    construct and return the appropriate Profile object
    based on the supplied keyword argument. If no profile keyword
    is supplied, it defaults to a basic Profile. This also requires
    that you pass through all the relevant keyword arguments for
    the constructors to the Profile objects and the objects that
    inherit from Profile.

    Parameters
    ----------
    Mandatory Keywords

    pres : array_like
        The pressure values (Hectopascals)
    hght : array_like
        The corresponding height values (Meters)
    tmpc : array_like
        The corresponding temperature values (Celsius)
    dwpc : array_like
        The corresponding dewpoint temperature values (Celsius)

    Optional Keyword Pairs (must use one or the other)

    wdir : array_like
        The direction from which the wind is blowing in meteorological degrees
    wspd : array_like
        The speed of the wind (kts)

    OR

    u : array_like
        The U-component of the direction from which the wind is blowing. (kts)

    v : array_like
        The V-component of the direction from which the wind is blowing. (kts)

    Optional Keywords

    missing : number, optional (default: sharppy.sharptab.constants.MISSING)
        The value of the missing flag used in the Profile objects

    profile : string, optional (default: 'default')
        The text identifier for the Profile to be generated. Valid options
        include ('default' | 'raw' | 'convective'). Default will construct a basic
        Profile, and convective will construct a ConvectiveProfile used for
        the SPC style GUI.

    omeg: array_like
        The corresponding vertical velocity values (Pa/s)

    Returns
    -------

    Profile : a basic Profile object
        This is the most basic and default object.

    OR

    ConvectiveProfile : a child of Profile
        This is the class used for the SPC GUI.


    '''
    ## Get the user's input for which Profile to construct.
    ## Make the default the 'default' profile.
    profile = kwargs.get('profile', 'default')

    ## if the profile is default, pass the rest of the keyword
    ## arguments to the BasicProfile object and return it
    if profile == 'default':
        return BasicProfile(**kwargs)
    ## if the profile is raw, return a base profile object
    elif profile == 'raw':
        return Profile(**kwargs)
    ## if the profile is convective, pass the rest of the keyword
    ## arguments to the ConvectiveProfile object and return it
    elif profile == 'convective':
        return ConvectiveProfile(**kwargs)

class Profile(object):
    def __init__(self, **kwargs):
        ## set the missing variable
        self.missing = kwargs.get('missing', MISSING)
        self.profile = kwargs.get('profile')
        self.latitude = kwargs.get('latitude', ma.masked)
        self.strictQC = kwargs.get('strictQC', False)

        # JTS - Get the cloud top fraction/pressure values.
        self.ctf_low = kwargs.get('ctf_low')
        self.ctf_high = kwargs.get('ctf_high')
        self.ctp_low = kwargs.get('ctp_low')
        self.ctp_high = kwargs.get('ctp_high')

        ## get the data and turn them into arrays
        self.pres = ma.asanyarray(kwargs.get('pres'), dtype=float)
        self.hght = ma.asanyarray(kwargs.get('hght'), dtype=float)
        self.tmpc = ma.asanyarray(kwargs.get('tmpc'), dtype=float)
        self.dwpc = ma.asanyarray(kwargs.get('dwpc'), dtype=float)

        assert self.pres.ndim == 1 and self.hght.ndim == 1 and self.tmpc.ndim == 1 and self.dwpc.ndim == 1,\
               "The dimensions of the pres, hght, tmpc, and dwpc arrays passed to the Profile object constructor are not all one dimensional."

        assert len(self.pres) > 1 and len(self.hght) > 1 and len(self.tmpc) > 1 and len(self.dwpc) > 1,\
               "The length of the pres, hght, tmpc, and dwpc arrays passed to Profile object constructor must all have a length greater than 1."

        assert len(self.pres) == len(self.hght) == len(self.tmpc) == len(self.dwpc),\
                "The pres, hght, tmpc, or dwpc arrays passed to the Profile object constructor must all have the same length."

        if np.ma.max(self.pres) <= 100:
            warnings.warn("The pressure values passed to the profile object are below 100 mb.  This may cause some the SHARPpy routines not to behave as expected.")

        if 'wdir' in kwargs and 'wspd' in kwargs:
            self.wdir = ma.asanyarray(kwargs.get('wdir'), dtype=float)
            self.wspd = ma.asanyarray(kwargs.get('wspd'), dtype=float)
            assert len(self.wdir) == len(self.wspd) == len(self.pres), "The wdir and wspd arrays passed to the Profile constructor must have the same length as the pres array."
            assert self.wdir.ndim == 1 and self.wspd.ndim == 1, "The wdir and wspd arrays passed to the Profile constructor are not one dimensional."
            # self.u, self.v = utils.vec2comp(self.wdir, self.wspd)
            self.u = None
            self.v = None
        ## did the user provide the wind in u,v form?
        elif 'u' in kwargs and 'v' in kwargs:
            self.u = ma.asanyarray(kwargs.get('u'), dtype=float)
            self.v = ma.asanyarray(kwargs.get('v'), dtype=float)
            assert len(self.u) == len(self.v) == len(self.pres), "The u and v arrays passed to the Profile constructor must have the same length as the pres array."
            assert self.u.ndim == 1 and self.v.ndim == 1, "The u and v arrays passed to the Profile constructor are not one dimensional."
            # self.wdir, self.wspd = utils.comp2vec(self.u, self.v)
            self.wdir = None
            self.wspd = None
        else:
            warnings.warn("No wind data (wdir/wspd or u/v) passed to the Profile object constructor.  This may cause some of the SHARPpy routines to not behave as expected.")

        ## check if any standard deviation data was supplied
        if 'tmp_stdev' in kwargs:
            self.dew_stdev = ma.asanyarray(kwargs.get('dew_stdev'), dtype=float)
            self.tmp_stdev = ma.asanyarray(kwargs.get('tmp_stdev'), dtype=float)
        else:
            self.dew_stdev = None
            self.tmp_stdev = None

        if kwargs.get('omeg', None) is not None:
            ## get the omega data and turn into arrays
            self.omeg = ma.asanyarray(kwargs.get('omeg'))
            assert len(self.omeg) == len(self.pres), "Length of omeg array passed to constructor is not the same length as the pres array."
            assert self.omeg.ndim == 1, "omeg array is not one dimensional."
            assert len(self.omeg) > 1, "omeg array length must have a length greater than 1."
        else:
            self.omeg = None

        ## optional keyword argument for location
        self.location = kwargs.get('location', None)
        self.date = kwargs.get('date', None)

        if self.strictQC is True:
            self.checkDataIntegrity()

    @classmethod
    def copy(cls, prof, strictQC=False, **kwargs):
        '''
            Copies a profile object.
        '''
        # JTS - Add the cloud top variables to the keyword argument list.
        new_kwargs = dict( (k, prof.__dict__[k]) for k in [ 'pres', 'hght', 'tmpc', 'dwpc', 'omeg', 'location', 'date', 'latitude', 'strictQC', 'missing', \
                                                            'ctf_low', 'ctf_high', 'ctp_low', 'ctp_high'])

        if prof.u is not None and prof.v is not None:
            new_kwargs.update({'u':prof.u, 'v':prof.v})
        else:
            new_kwargs.update({'wspd':prof.wspd, 'wdir':prof.wdir})

        new_kwargs.update({'strictQC':strictQC})

        # Create a new profile object using the old profile object data cls is the Class type (e.g., ConvectiveProfile)
        new_kwargs.update(kwargs)
        new_prof = cls(**new_kwargs)

        if hasattr(prof, 'srwind'):
            rmu, rmv, lmu, lmv = prof.srwind
            new_prof.set_srright(rmu, rmv)
            new_prof.set_srleft(lmu, lmv)

        return new_prof

    def toFile(self, file_name):
        snd_file = open(file_name, 'w')
        def qc(val):
            return -9999. if not utils.QC(val) else val

        snd_loc = (" " * (4 - len(self.location))) + self.location

        now = datetime.utcnow()
        #print(now, self.date)
        user = getpass.getuser()
        snd_file.write("%TITLE%\n")
        snd_file.write("%s   %s\n Saved by user: %s on %s UTC\n" % (snd_loc, self.date.strftime("%y%m%d/%H%M"), user, now.strftime('%Y%m%d/%H%M')))
        snd_file.write("   LEVEL       HGHT       TEMP       DWPT       WDIR       WSPD\n")
        snd_file.write("-------------------------------------------------------------------\n")
        snd_file.write("%RAW%\n")
        for idx in range(self.pres.shape[0]):
            str = ""
            for col in ['pres', 'hght', 'tmpc', 'dwpc', 'wdir', 'wspd']:
                str += "%8.2f,  " % qc(self.__dict__[col][idx])

            snd_file.write(str[:-3] + "\n")
        snd_file.write("%END%\n")
        snd_file.close()

    def checkDataIntegrity(self):
        if not isHGHTValid(self.hght):
            raiseError("Invalid height data.  Data has repeat height values or height does not increase as pressure decreases.", DataQualityException)
        if not isTMPCValid(self.tmpc):
            raiseError("Invalid temperature data. Profile contains a temperature value < -273.15 Celsius.", DataQualityException)
        if not isDWPCValid(self.dwpc):
            raiseError("Invalid dewpoint data. Profile contains a dewpoint value < -273.15 Celsius.", DataQualityException)
        if not isWSPDValid(self.wspd):
            raiseError("Invalid wind speed data. Profile contains a wind speed value < 0 knots.", DataQualityException)
        if not isWDIRValid(self.wdir):
            raiseError("Invalid wind direction data. Profile contains a wind direction < 0 degrees or >= 360 degrees.", DataQualityException)


class BasicProfile(Profile):
    '''
    The default data class for SHARPpy.
    All other data classes inherit from this class.
    This class holds the vertical data for pressure,
    height, temperature, dewpoint, and winds. This class
    has no indices computed.

    '''

    def __init__(self, **kwargs):
        '''
        Create the sounding data object

        Parameters
        ----------
        Mandatory Keywords
        pres : array_like
            The pressure values (Hectopaschals)
        hght : array_like
            The corresponding height values (Meters)
        tmpc : array_like
            The corresponding temperature values (Celsius)
        dwpc : array_like
        The corresponding dewpoint temperature values (Celsius)

        Optional Keyword Pairs (must use one or the other)
        wdir : array_like
            The direction from which the wind is blowing in
            meteorological degrees
        wspd : array_like
            The speed of the wind (kts)

        OR

        u : array_like
            The U-component of the direction from which the wind
            is blowing (kts)

        v : array_like
            The V-component of the direction from which the wind
            is blowing. (kts)

        Optional Keywords
        missing : number (default: sharppy.sharptab.constants.MISSING)
            The value of the missing flag

        location : string (default: None)
            The 3 character station identifier or 4 character
            WMO station ID for radiosonde locations. Used for
            the PWV database.

        strictQC : boolean
            A flag that indicates whether or not the strict quality control
            routines should be run on the profile upon construction.

        Returns
        -------
        prof: Profile object

        '''
        super(BasicProfile, self).__init__(**kwargs)

        self.strictQC = kwargs.get('strictQC', True)

        ## did the user provide the wind in vector form?
        if self.wdir is not None:
            self.wdir[self.wdir == self.missing] = ma.masked
            self.wspd[self.wspd == self.missing] = ma.masked
            self.wdir[self.wspd.mask] = ma.masked
            self.wspd[self.wdir.mask] = ma.masked
            self.u, self.v = utils.vec2comp(self.wdir, self.wspd)
        ## did the user provide the wind in u,v form?
        elif self.u is not None:
            self.u[self.u == self.missing] = ma.masked
            self.v[self.v == self.missing] = ma.masked
            self.u[self.v.mask] = ma.masked
            self.v[self.u.mask] = ma.masked
            self.wdir, self.wspd = utils.comp2vec(self.u, self.v)

        ## check if any standard deviation data was supplied
        if self.tmp_stdev is not None:
            self.dew_stdev[self.dew_stdev == self.missing] = ma.masked
            self.tmp_stdev[self.tmp_stdev == self.missing] = ma.masked
            self.dew_stdev.set_fill_value(self.missing)
            self.tmp_stdev.set_fill_value(self.missing)

        if self.omeg is not None:
            ## get the omega data and turn into arrays
            self.omeg[self.omeg == self.missing] = ma.masked
        else:
            self.omeg = ma.masked_all(len(self.hght))

        # QC Checks on the arrays passed to the constructor.
        areProfileArrayLengthEqual(self)

        ## mask the missing values
        self.pres[self.pres == self.missing] = ma.masked
        self.hght[self.hght == self.missing] = ma.masked
        self.tmpc[self.tmpc == self.missing] = ma.masked
        self.dwpc[self.dwpc == self.missing] = ma.masked

        self.logp = np.log10(self.pres.copy())
        self.vtmp = thermo.virtemp( self.pres, self.tmpc, self.dwpc )
        idx = np.ma.where(self.pres > 0)[0]
        self.vtmp[self.dwpc.mask[idx]] = self.tmpc[self.dwpc.mask[idx]] # Masking any virtual temperature

        ## get the index of the top and bottom of the profile
        self.sfc = self.get_sfc()
        self.top = self.get_top()

        if self.strictQC is True:
            self.checkDataIntegrity()

        ## generate the wetbulb profile
        self.wetbulb = self.get_wetbulb_profile()
        ## generate theta-e profile
        self.thetae = self.get_thetae_profile()
        ## generate theta profile
        self.theta = self.get_theta_profile()
        ## generate water vapor mixing ratio profile
        self.wvmr = self.get_wvmr_profile()
        ## generate rh profile
        self.relh = self.get_rh_profile()

    def get_sfc(self):
        '''
            Convenience function to get the index of the surface. It is
            determined by finding the lowest level in which a temperature is
            reported.

            Parameters
            ----------
            None

            Returns
            -------
            Index of the surface

            '''
        return np.where(~self.tmpc.mask)[0].min()

    def get_top(self):
        '''
            Convenience function to get the index of the surface. It is
            determined by finding the lowest level in which a temperature is
            reported.

            Parameters
            ----------
            None

            Returns
            -------
            Index of the surface
            '''
        return np.where(~self.tmpc.mask)[0].max()

    def get_wvmr_profile(self):
        '''
            Function to calculate the water vapor mixing ratio profile.

            Parameters
            ----------
            None

            Returns
            -------
            Array of water vapor mixing ratio profile
            '''

        #wvmr = ma.empty(self.pres.shape[0])
        #for i in range(len(self.v)):
        wvmr = thermo.mixratio( self.pres, self.dwpc )
        wvmr[wvmr == self.missing] = ma.masked
        wvmr.set_fill_value(self.missing)
        return wvmr

    def get_wetbulb_profile(self):
        '''
            Function to calculate the wetbulb profile.

            Parameters
            ----------
            None

            Returns
            -------
            Array of wet bulb profile
            '''

        wetbulb = ma.empty(self.pres.shape[0])
        for i in range(len(self.v)):
            wetbulb[i] = thermo.wetbulb( self.pres[i], self.tmpc[i], self.dwpc[i] )
        wetbulb[wetbulb == self.missing] = ma.masked
        wetbulb.set_fill_value(self.missing)
        return wetbulb

    def get_theta_profile(self):
        '''
            Function to calculate the theta profile.

            Parameters
            ----------
            None

            Returns
            -------
            Array of theta profile
            '''
        theta = ma.empty(self.pres.shape[0])
        for i in range(len(self.v)):
            theta[i] = thermo.theta(self.pres[i], self.tmpc[i])
        theta[theta == self.missing] = ma.masked
        theta.set_fill_value(self.missing)
        theta = thermo.ctok(theta)
        return theta

    def get_thetae_profile(self):
        '''
            Function to calculate the theta-e profile.

            Parameters
            ----------
            None

            Returns
            -------
            Array of theta-e profile
            '''
        thetae = ma.empty(self.pres.shape[0])
        for i in range(len(self.v)):
            thetae[i] = thermo.ctok( thermo.thetae(self.pres[i], self.tmpc[i], self.dwpc[i]) )
        thetae[thetae == self.missing] = ma.masked
        thetae.set_fill_value(self.missing)
        return thetae

    def get_rh_profile(self):
        '''
        Function to calculate the relative humidity profile

        Parameters
        ----------
        None

        Returns
        -------
        Array of the relative humidity profile
        '''

        rh = thermo.relh(self.pres, self.tmpc, self.dwpc)
        rh[rh == self.missing] = ma.masked
        rh.set_fill_value(self.missing)
        return rh


class ConvectiveProfile(BasicProfile):
    '''
    The Convective data class for SHARPPy. This is the class used
    to generate the indices that are default for the SPC NSHARP display.

    This class inherits from the Profile object.

    '''
    def __init__(self, **kwargs):
        '''
        Create the sounding data object

        Parameters
        ----------
        Mandatory Keywords
        pres : array_like
            The pressure values (Hectopaschals)
        hght : array_like
            The corresponding height values (Meters)
        tmpc : array_like
            The corresponding temperature values (Celsius)
        dwpc : array_like
            The corresponding dewpoint temperature values (Celsius)

        Optional Keyword Pairs (must use one or the other)
        wdir : array_like
            The direction from which the wind is blowing in
            meteorological degrees
        wspd : array_like
            The speed of the wind (kts)

        OR

        u : array_like
            The U-component of the direction from which the wind
            is blowing

        v : array_like
            The V-component of the direction from which the wind
            is blowing.

        missing : number, optional (default: sharppy.sharptab.constants.MISSING)
            The value of the missing flag

        location : string, optional (default: None)
            The 3 character station identifier or 4 character
            WMO station ID for radiosonde locations. Used for
            the PWV database.

        omeg : array_like, optional
            List of the vertical velocity in pressure coordinates with height (Pascals/second)

        Returns
        -------
        A profile object
        '''
        ## call the constructor for Profile
        super(ConvectiveProfile, self).__init__(**kwargs)
        assert np.ma.max(self.pres) > 100, "ConvectiveProfile objects require that the minimum pressure passed in the data array is greater than 100 mb."

        self.user_srwind = None

        # Generate the fire weather paramters
        logging.debug("Calling get_fire().")
        dt = datetime.now()
        self.get_fire()
        logging.debug("get_fire() took: " + str((datetime.now() - dt)))

        # Generate the winter inset/precipitation types
        logging.debug("Calling get_precip().")
        dt = datetime.now()
        self.get_precip()
        logging.debug("get_precip() took: " + str((datetime.now() - dt)))

        ## generate various parcels
        logging.debug("Calling get_parcels().")
        dt = datetime.now()
        self.get_parcels()
        logging.debug("get_parcels() took: " + str((datetime.now() - dt)))

        ## calculate thermodynamic window indices
        logging.debug("Calling get_thermo().")
        dt = datetime.now()
        self.get_thermo()
        logging.debug("get_thermo() took: " + str((datetime.now() - dt)))

        ## generate wind indices
        logging.debug("Calling get_kinematics().")
        dt = datetime.now()
        self.get_kinematics()
        logging.debug("get_kinematics() took: " + str((datetime.now() - dt)))

        ## get SCP, STP(cin), STP(fixed), SHIP
        logging.debug("Calling get_severe().")
        dt = datetime.now()
        self.get_severe()
        logging.debug("get_severe() took: " + str((datetime.now() - dt)))

        ## calculate the SARS database matches
        logging.debug("Calling get_sars().")
        dt = datetime.now()
        self.get_sars()
        logging.debug("get_sars() took: " + str((datetime.now() - dt)))

        ## get the precipitable water climatology
        logging.debug("Calling get_PWV_loc().")
        dt = datetime.now()
        self.get_PWV_loc()
        logging.debug("get_PWV_loc() took: " + str((datetime.now() - dt)))

        ## get the parcel trajectory
        logging.debug("Calling get_traj().")
        dt = datetime.now()
        self.get_traj()
        logging.debug("get_traj() took: " + str((datetime.now() - dt)))

        ## miscellaneous indices I didn't know where to put
        logging.debug("Calling get_indices().")
        dt = datetime.now()
        self.get_indices()
        logging.debug("get_indices() took: " + str((datetime.now() - dt)))

        ## get the possible watch type
        logging.debug("Calling get_watch().")
        dt = datetime.now()
        self.get_watch()
        logging.debug("get_watch() took: " + str((datetime.now() - dt)))

    def get_fire(self):
        '''
        Function to generate different indices and information
        regarding any fire weather in the sounding.  This helps fill
        the data shown in the FIRE inset.

        Parameters
        ----------
        None

        Returns
        -------
        None
        '''
        self.fosberg = fire.fosberg(self)
        self.haines_hght = fire.haines_height(self)
        self.haines_low = fire.haines_low(self)
        self.haines_mid = fire.haines_mid(self)
        self.haines_high = fire.haines_high(self)
        self.ppbl_top = params.pbl_top(self)
        self.sfc_rh = thermo.relh(self.pres[self.sfc], self.tmpc[self.sfc], self.dwpc[self.sfc])
        pres_sfc = self.pres[self.sfc]
        pres_1km = interp.pres(self, interp.to_msl(self, 1000.))
        self.pbl_h = interp.to_agl(self, interp.hght(self, self.ppbl_top))
        self.rh01km = params.mean_relh(self, pbot=pres_sfc, ptop=pres_1km)
        self.pblrh = params.mean_relh(self, pbot=pres_sfc, ptop=self.ppbl_top)
        self.meanwind01km = winds.mean_wind(self, pbot=pres_sfc, ptop=pres_1km)
        self.meanwindpbl = winds.mean_wind(self, pbot=pres_sfc, ptop=self.ppbl_top)
        self.pblmaxwind = winds.max_wind(self, lower=0, upper=self.pbl_h)
        #self.pblmaxwind = [np.ma.masked, np.ma.masked]
        mulplvals = params.DefineParcel(self, flag=3, pres=500)
        mupcl = params.cape(self, lplvals=mulplvals)
        self.bplus_fire = mupcl.bplus

    def get_precip(self):
        '''
        Function to generate different indices and information
        regarding any precipitation in the sounding.  This helps fill
        the data shown in the WINTER inset.

        Returns nothing, but sets the following
        variables:

        self.dgz_pbot, self.dgz_ptop : the dendretic growth zone (DGZ) top and bottom (mb)
        self.dgz_meanrh : DGZ mean relative humidity (%)
        self.dgz_pw : the preciptable water vapor in the DGZ (inches)
        self.dgz_meanq : the mean water vapor mixing ratio in the DGZ (g/kg)
        self.dgz_meanomeg : the mean omega in the DGZ (microbars/second)
        self.oprh : the OPRH variable (units don't mean anything)
        self.plevel, self.phase, self.tmp, self.st : the initial phase, level, temperature, and state of any precip in the sounding
        self.tpos, self.tneg, self.ttop, self.tbot : positive and negative temperature layers in the sounding
        self.wpos, self.wneg, self.wtop, self.wbot : positive and negative wetbulb layers in the soundings
        self.precip_type : the best guess precipitation type

        Parameters
        ----------
        None

        Returns
        -------
        None
        '''
        self.dgz_pbot, self.dgz_ptop = params.dgz(self)
        self.dgz_meanrh = params.mean_relh(self, pbot=self.dgz_pbot, ptop=self.dgz_ptop)
        self.dgz_pw = params.precip_water(self, pbot=self.dgz_pbot, ptop=self.dgz_ptop)
        self.dgz_meanq = params.mean_mixratio(self, pbot=self.dgz_pbot, ptop=self.dgz_ptop)
        self.dgz_meanomeg = params.mean_omega(self, pbot=self.dgz_pbot, ptop=self.dgz_ptop) * 10 # to microbars/sec
        self.oprh = self.dgz_meanomeg * self.dgz_pw * (self.dgz_meanrh/100.)

        self.plevel, self.phase, self.tmp, self.st = watch_type.init_phase(self)
        self.tpos, self.tneg, self.ttop, self.tbot = watch_type.posneg_temperature(self, start=self.plevel)
        self.wpos, self.wneg, self.wtop, self.wbot = watch_type.posneg_wetbulb(self, start=self.plevel)
        self.precip_type = watch_type.best_guess_precip(self, self.phase, self.plevel, self.tmp, self.tpos, self.tneg)


    def get_parcels(self):
        '''
        Function to generate various parcels and parcel
        traces.
        Returns nothing, but sets the following
        variables:

        self.mupcl : Most Unstable Parcel
        self.sfcpcl : Surface Based Parcel
        self.mlpcl : Mixed Layer Parcel
        self.fcstpcl : Forecast Surface Parcel
        self.ebottom : The bottom pressure level of the effective inflow layer
        self.etop : the top pressure level of the effective inflow layer
        self.ebotm : The bottom, meters (agl), of the effective inflow layer
        self.etopm : The top, meters (agl), of the effective inflow layer

        Parameters
        ----------
        None

        Returns
        -------
        None
        '''

        self.mupcl = params.parcelx( self, flag=3 )
        if self.mupcl.lplvals.pres == self.pres[self.sfc]:
            self.sfcpcl = self.mupcl
        else:
            self.sfcpcl = params.parcelx( self, flag=1 )
        self.fcstpcl = params.parcelx( self, flag=2 )
        self.mlpcl = params.parcelx( self, flag=4 )
        self.usrpcl = params.Parcel()

        ## get the effective inflow layer data
        self.ebottom, self.etop = params.effective_inflow_layer( self, mupcl=self.mupcl )

        ## if there was no effective inflow layer, set the values to masked
        if self.etop is ma.masked or self.ebottom is ma.masked:
            self.ebotm = ma.masked; self.etopm = ma.masked
            self.effpcl = self.sfcpcl # Default to surface parcel, as in params.DefineProfile().

        ## otherwise, interpolate the heights given to above ground level
        else:
            self.ebotm = interp.to_agl(self, interp.hght(self, self.ebottom))
            self.etopm = interp.to_agl(self, interp.hght(self, self.etop))
            # The below code was adapted from params.DefineProfile()
            # Lifting one additional parcel probably won't slow the program too much.
            # It's just one more lift compared to all the lifts in the params.effective_inflow_layer() call.
            mtha = params.mean_theta(self, self.ebottom, self.etop)
            mmr = params.mean_mixratio(self, self.ebottom, self.etop)
            effpres = (self.ebottom+self.etop)/2.
            efftmpc = thermo.theta(1000., mtha, effpres)
            effdwpc = thermo.temp_at_mixrat(mmr, effpres)
            self.effpcl = params.parcelx(self, flag=5, pres=effpres, tmpc=efftmpc, dwpc=effdwpc) #This is the effective parcel.

    def get_kinematics(self):
        '''
        Function to generate the numerous kinematic quantities
        used for display and calculations. It requires that the
        parcel calculations have already been called for the lcl
        to el shear and mean wind vectors, as well as indices
        that require an effective inflow layer.

        Parameters
        ----------
        None

        Returns
        -------
        None
        '''
        sfc = self.pres[self.sfc]
        heights = np.array([1000., 3000., 4000., 5000., 6000., 8000., 9000.])
        p1km, p3km, p4km, p5km, p6km, p8km, p9km = interp.pres(self, interp.to_msl(self, heights))
        ## 1km and 6km winds
        self.wind1km = interp.vec(self, p1km)
        self.wind6km = interp.vec(self, p6km)
        ## calcluate wind shear
        self.sfc_1km_shear = winds.wind_shear(self, pbot=sfc, ptop=p1km)
        self.sfc_3km_shear = winds.wind_shear(self, pbot=sfc, ptop=p3km)
        self.sfc_6km_shear = winds.wind_shear(self, pbot=sfc, ptop=p6km)
        self.sfc_8km_shear = winds.wind_shear(self, pbot=sfc, ptop=p8km)
        self.sfc_9km_shear = winds.wind_shear(self, pbot=sfc, ptop=p9km)
        self.lcl_el_shear = winds.wind_shear(self, pbot=self.mupcl.lclpres, ptop=self.mupcl.elpres)
        ## calculate mean wind
        self.mean_1km = utils.comp2vec(*winds.mean_wind(self, pbot=sfc, ptop=p1km))
        self.mean_3km = utils.comp2vec(*winds.mean_wind(self, pbot=sfc, ptop=p3km))
        self.mean_6km = utils.comp2vec(*winds.mean_wind(self, pbot=sfc, ptop=p6km))
        self.mean_8km = utils.comp2vec(*winds.mean_wind(self, pbot=sfc, ptop=p8km))
        self.mean_lcl_el = utils.comp2vec(*winds.mean_wind(self, pbot=self.mupcl.lclpres, ptop=self.mupcl.elpres))
        ## parameters that depend on the presence of an effective inflow layer
        if self.etop is ma.masked or self.ebottom is ma.masked:
            self.etopm = ma.masked; self.ebotm = ma.masked
            self.bunkers = winds.non_parcel_bunkers_motion( self )
            if self.user_srwind is None:
                self.user_srwind = self.bunkers
            self.srwind = self.user_srwind
            self.eff_shear = [MISSING, MISSING]
            self.ebwd = [MISSING, MISSING, MISSING]
            self.ebwspd = MISSING
            self.mean_eff = [MISSING, MISSING, MISSING]
            self.mean_ebw = [MISSING, MISSING, MISSING]

            self.right_srw_eff = [MISSING, MISSING, MISSING]
            self.right_srw_ebw = [MISSING, MISSING, MISSING]
            self.right_esrh = [ma.masked, ma.masked, ma.masked]
            self.right_critical_angle = ma.masked

            self.left_srw_eff = [MISSING, MISSING, MISSING]
            self.left_srw_ebw = [MISSING, MISSING, MISSING]
            self.left_esrh = [ma.masked, ma.masked, ma.masked]
            self.left_critical_angle = ma.masked
        else:
            self.bunkers = params.bunkers_storm_motion(self, mupcl=self.mupcl, pbot=self.ebottom)
            if self.user_srwind is None:
                self.user_srwind = self.bunkers
            self.srwind = self.user_srwind
            depth = ( self.mupcl.elhght - self.ebotm ) / 2
            elh = interp.pres(self, interp.to_msl(self, self.ebotm + depth))
            ## calculate mean wind
            self.mean_eff = winds.mean_wind(self, self.ebottom, self.etop )
            self.mean_ebw = winds.mean_wind(self, pbot=self.ebottom, ptop=elh )
            ## calculate wind shear of the effective layer
            self.eff_shear = winds.wind_shear(self, pbot=self.ebottom, ptop=self.etop)
            self.ebwd = winds.wind_shear(self, pbot=self.ebottom, ptop=elh)
            self.ebwspd = utils.mag( self.ebwd[0], self.ebwd[1] )
            ## calculate quantities relative to the right-mover vector
            self.right_srw_eff = winds.sr_wind(self, pbot=self.ebottom, ptop=self.etop, stu=self.srwind[0], stv=self.srwind[1] )
            self.right_srw_ebw = winds.sr_wind(self, pbot=self.ebottom, ptop=elh, stu=self.srwind[0], stv=self.srwind[1] )
            self.right_esrh = winds.helicity(self, self.ebotm, self.etopm, stu=self.srwind[0], stv=self.srwind[1])
            self.right_critical_angle = winds.critical_angle(self, stu=self.srwind[0], stv=self.srwind[1])
            ## calculate quantities relative to the left-mover vector
            self.left_srw_eff = winds.sr_wind(self, pbot=self.ebottom, ptop=self.etop, stu=self.srwind[2], stv=self.srwind[3] )
            self.left_srw_ebw = winds.sr_wind(self, pbot=self.ebottom, ptop=elh, stu=self.srwind[2], stv=self.srwind[3] )
            self.left_esrh = winds.helicity(self, self.ebotm, self.etopm, stu=self.srwind[2], stv=self.srwind[3])
            self.left_critical_angle = winds.critical_angle(self, stu=self.srwind[2], stv=self.srwind[3])

        ## calculate quantities relative to the right-mover vector
        self.right_srw_1km = utils.comp2vec(*winds.sr_wind(self, pbot=sfc, ptop=p1km, stu=self.srwind[0], stv=self.srwind[1] ))
        self.right_srw_3km = utils.comp2vec(*winds.sr_wind(self, pbot=sfc, ptop=p3km, stu=self.srwind[0], stv=self.srwind[1] ))
        self.right_srw_6km = utils.comp2vec(*winds.sr_wind(self, pbot=sfc, ptop=p6km, stu=self.srwind[0], stv=self.srwind[1] ))
        self.right_srw_8km = utils.comp2vec(*winds.sr_wind(self, pbot=sfc, ptop=p8km, stu=self.srwind[0], stv=self.srwind[1] ))
        self.right_srw_4_5km = utils.comp2vec(*winds.sr_wind(self, pbot=p4km, ptop=p5km, stu=self.srwind[0], stv=self.srwind[1] ))
        self.right_srw_lcl_el = utils.comp2vec(*winds.sr_wind(self, pbot=self.mupcl.lclpres, ptop=self.mupcl.elpres, stu=self.srwind[0], stv=self.srwind[1] ))
        # This is for the red, blue, and purple bars that appear on the SR Winds vs. Height plot
        self.right_srw_0_2km = winds.sr_wind(self, pbot=sfc, ptop=interp.pres(self, interp.to_msl(self, 2000.)), stu=self.srwind[0], stv=self.srwind[1])
        self.right_srw_4_6km = winds.sr_wind(self, pbot=interp.pres(self, interp.to_msl(self, 4000.)), ptop=p6km, stu=self.srwind[0], stv=self.srwind[1])
        self.right_srw_9_11km = winds.sr_wind(self, pbot=interp.pres(self, interp.to_msl(self, 9000.)), ptop=interp.pres(self, interp.to_msl(self, 11000.)), stu=self.srwind[0], stv=self.srwind[1])

        ## calculate quantities relative to the left-mover vector
        self.left_srw_1km = utils.comp2vec(*winds.sr_wind(self, pbot=sfc, ptop=p1km, stu=self.srwind[2], stv=self.srwind[3] ))
        self.left_srw_3km = utils.comp2vec(*winds.sr_wind(self, pbot=sfc, ptop=p3km, stu=self.srwind[2], stv=self.srwind[3] ))
        self.left_srw_6km = utils.comp2vec(*winds.sr_wind(self, pbot=sfc, ptop=p6km, stu=self.srwind[2], stv=self.srwind[3] ))
        self.left_srw_8km = utils.comp2vec(*winds.sr_wind(self, pbot=sfc, ptop=p8km, stu=self.srwind[2], stv=self.srwind[3] ))
        self.left_srw_4_5km = utils.comp2vec(*winds.sr_wind(self, pbot=p4km, ptop=p5km, stu=self.srwind[2], stv=self.srwind[3] ))
        self.left_srw_lcl_el = utils.comp2vec(*winds.sr_wind(self, pbot=self.mupcl.lclpres, ptop=self.mupcl.elpres, stu=self.srwind[2], stv=self.srwind[3] ))
        # This is for the red, blue, and purple bars that appear on the SR Winds vs. Height plot
        self.left_srw_0_2km = winds.sr_wind(self, pbot=sfc, ptop=interp.pres(self, interp.to_msl(self, 2000.)), stu=self.srwind[2], stv=self.srwind[3])
        self.left_srw_4_6km = winds.sr_wind(self, pbot=interp.pres(self, interp.to_msl(self, 4000.)), ptop=p6km, stu=self.srwind[2], stv=self.srwind[3])
        self.left_srw_9_11km = winds.sr_wind(self, pbot=interp.pres(self, interp.to_msl(self, 9000.)), ptop=interp.pres(self, interp.to_msl(self, 11000.)), stu=self.srwind[2], stv=self.srwind[3])

        ## calculate upshear and downshear
        self.upshear_downshear = winds.mbe_vectors(self)
        self.right_srh1km = winds.helicity(self, 0, 1000., stu=self.srwind[0], stv=self.srwind[1])
        self.right_srh3km = winds.helicity(self, 0, 3000., stu=self.srwind[0], stv=self.srwind[1])
        self.left_srh1km = winds.helicity(self, 0, 1000., stu=self.srwind[2], stv=self.srwind[3])
        self.left_srh3km = winds.helicity(self, 0, 3000., stu=self.srwind[2], stv=self.srwind[3])

        self.srw_eff = self.right_srw_eff
        self.srw_ebw = self.right_srw_ebw
        self.esrh = self.right_esrh
        self.critical_angle = self.right_critical_angle
        self.srw_1km = self.right_srw_1km
        self.srw_3km = self.right_srw_3km
        self.srw_6km = self.right_srw_6km
        self.srw_8km = self.right_srw_8km
        self.srw_4_5km = self.right_srw_4_5km
        self.srw_lcl_el = self.right_srw_lcl_el
        self.srw_0_2km = self.right_srw_0_2km
        self.srw_4_6km = self.right_srw_4_6km
        self.srw_9_11km = self.right_srw_9_11km
        self.srh1km = self.right_srh1km
        self.srh3km = self.right_srh3km

    def get_thermo(self):
        '''
        Function to generate thermodynamic indices.

        Function returns nothing, but sets the following
        variables:

        self.k_idx - K Index, a severe weather index
        self.pwat - Precipitable Water Vapor (inches)
        self.lapserate_3km - 0 to 3km AGL lapse rate (C/km)
        self.lapserate_3_6km - 3 to 6km AGL lapse rate (C/km)
        self.lapserate_850_500 - 850 to 500mb lapse rate (C/km)
        self.lapserate_700_500 - 700 to 500mb lapse rate (C/km)
        self.convT - The Convective Temperature (F)
        self.maxT - The Maximum Forecast Surface Temp (F)
        self.mean_mixr - Mean Mixing Ratio
        self.low_rh - low level mean relative humidity
        self.mid_rh - mid level mean relative humidity
        self.totals_totals - Totals Totals index, a severe weather index

        Parameters
        ----------
        None

        Returns
        -------
        None
        '''
        ## either get or calculate the indices, round to the nearest int, and
        ## convert them to strings.
        ## K Index
        self.k_idx = params.k_index( self )
        ## precipitable water
        self.pwat = params.precip_water( self )
        ## 0-3km agl lapse rate
        self.lapserate_3km = params.lapse_rate( self, 0., 3000., pres=False )
        ## 3-6km agl lapse rate
        self.lapserate_3_6km = params.lapse_rate( self, 3000., 6000., pres=False )
        ## 850-500mb lapse rate
        self.lapserate_850_500 = params.lapse_rate( self, 850., 500., pres=True )
        ## 700-500mb lapse rate
        self.lapserate_700_500 = params.lapse_rate( self, 700., 500., pres=True )
        ## 2-6 km max lapse rate
        self.max_lapse_rate_2_6 = params.max_lapse_rate( self )
        ## convective temperature
        self.convT = thermo.ctof( params.convective_temp( self ) )
        ## sounding forecast surface temperature
        self.maxT = thermo.ctof( params.max_temp( self ) )
        #fzl = str(int(self.sfcparcel.hght0c))
        ## 100mb mean mixing ratio
        self.mean_mixr = params.mean_mixratio( self )
        ## 150mb mean rh
        self.low_rh = params.mean_relh( self )
        self.mid_rh = params.mean_relh( self, pbot=(self.pres[self.sfc] - 150),
            ptop=(self.pres[self.sfc] - 350) )
        ## calculate the totals totals index
        self.totals_totals = params.t_totals( self )
        ## calculate the inferred temperature advection
        self.inf_temp_adv = params.inferred_temp_adv(self, lat=self.latitude)

    def get_severe(self):
        '''
        Function to calculate special severe weather indices.
        Requires calling get_parcels() and get_kinematics().

        Returns nothing, but sets the following variables:

        self.right_stp_fixed - fixed layer significant tornado parameter (computed with SRH relative to the right-mover vector)
        self.left_stp_fixed - fixed layer significant tornado parameter (computed with SRH relative to the left-mover vector)
        self.right_stp_cin - effective layer significant tornado parameter (computed with SRH relative to the right-mover vector)
        self.left_stp_cin - effective layer significant tornado parameter (computed with SRH relative to the left-mover vector)
        self.right_scp - right moving supercell composite parameter
        self.left_scp - left moving supercell composite parameter

        Parameters
        ----------
        None

        Returns
        -------
        None
        '''
        wspd = utils.mag(self.sfc_6km_shear[0], self.sfc_6km_shear[1])
        self.right_stp_fixed = params.stp_fixed(self.sfcpcl.bplus, self.sfcpcl.lclhght, self.right_srh1km[0], utils.KTS2MS(wspd))
        self.left_stp_fixed = params.stp_fixed(self.sfcpcl.bplus, self.sfcpcl.lclhght, self.left_srh1km[0], utils.KTS2MS(wspd))
        self.sherbe = params.sherb(self, effective=True)

        if self.etop is np.ma.masked or self.ebottom is np.ma.masked:
            self.right_scp = 0.0; self.left_scp = 0.0
            self.right_stp_cin = 0.0; self.left_stp_cin = 0.0
        else:
            self.right_scp = params.scp( self.mupcl.bplus, self.right_esrh[0], utils.KTS2MS(self.ebwspd))
            self.left_scp = params.scp( self.mupcl.bplus, self.left_esrh[0], utils.KTS2MS(self.ebwspd))

            right_esrh = self.right_esrh[0]
            left_esrh = self.left_esrh[0]

            if self.latitude < 0:
                right_esrh = -right_esrh
                left_esrh = -left_esrh

            self.right_stp_cin = params.stp_cin(self.mlpcl.bplus, right_esrh, utils.KTS2MS(self.ebwspd),
                self.mlpcl.lclhght, self.mlpcl.bminus)
            self.left_stp_cin = params.stp_cin(self.mlpcl.bplus, left_esrh, utils.KTS2MS(self.ebwspd),
                self.mlpcl.lclhght, self.mlpcl.bminus)

            if self.latitude < 0:
                self.right_stp_cin = -self.right_stp_cin
                self.left_stp_cin = -self.left_stp_cin

        if self.latitude < 0:
            self.stp_fixed = self.left_stp_fixed
            self.stp_cin = self.left_stp_cin
            self.scp = self.left_scp
        else:
            self.stp_fixed = self.right_stp_fixed
            self.stp_cin = self.right_stp_cin
            self.scp = self.right_scp

    def get_sars(self):
        '''
        Function to get the SARS analogues from the hail and
        supercell databases. Requires calling get_kinematics()
        and get_parcels() first. Also calculates the significant
        hail parameter.

        Function returns nothing, but sets the following variables:

        self.matches - the matches from SARS HAIL
        self.ship - significant hail parameter
        self.supercell_matches - the matches from SARS SUPERCELL

        Parameters
        ----------
        None

        Returns
        -------
        None
        '''
        sfc_6km_shear = utils.KTS2MS( utils.mag( self.sfc_6km_shear[0], self.sfc_6km_shear[1]) )
        sfc_3km_shear = utils.KTS2MS( utils.mag( self.sfc_3km_shear[0], self.sfc_3km_shear[1]) )
        sfc_9km_shear = utils.KTS2MS( utils.mag( self.sfc_9km_shear[0], self.sfc_9km_shear[1]) )
        h500t = interp.temp(self, 500.)
        lapse_rate = params.lapse_rate( self, 700., 500., pres=True )
        right_srh3km = self.right_srh3km[0]
        right_srh1km = self.right_srh1km[0]
        left_srh3km = self.left_srh3km[0]
        left_srh1km = self.left_srh1km[0]
        mucape = self.mupcl.bplus
        mlcape = self.mlpcl.bplus
        mllcl = self.mlpcl.lclhght
        mumr = thermo.mixratio(self.mupcl.pres, self.mupcl.dwpc)
        self.ship = params.ship(self)

        self.hail_database = 'sars_hail.txt'
        self.supercell_database = 'sars_supercell.txt'
        try:
            self.right_matches = hail(self.hail_database, mumr, mucape, h500t, lapse_rate, sfc_6km_shear,
                sfc_9km_shear, sfc_3km_shear, right_srh3km)
        except:
            self.right_matches = ([], [], 0, 0, 0)

        try:
            self.left_matches = hail(self.hail_database, mumr, mucape, h500t, lapse_rate, sfc_6km_shear,
                sfc_9km_shear, sfc_3km_shear, -left_srh3km)
        except:
            self.left_matches = ([], [], 0, 0, 0)

        try:
            self.right_supercell_matches = supercell(self.supercell_database, mlcape, mllcl, h500t, lapse_rate,
                utils.MS2KTS(sfc_6km_shear), right_srh1km, utils.MS2KTS(sfc_3km_shear), utils.MS2KTS(sfc_9km_shear),
                right_srh3km)
        except:
            self.right_supercell_matches = ([], [], 0, 0, 0)

        try:
            self.left_supercell_matches = supercell(self.supercell_database, mlcape, mllcl, h500t, lapse_rate,
                utils.MS2KTS(sfc_6km_shear), -left_srh1km, utils.MS2KTS(sfc_3km_shear), utils.MS2KTS(sfc_9km_shear),
                -left_srh3km)
        except Exception as e:
            self.left_supercell_matches = ([], [], 0, 0, 0)

        if self.latitude < 0:
            self.supercell_matches = self.left_supercell_matches
            self.matches = self.left_matches
        else:
            self.supercell_matches = self.right_supercell_matches
            self.matches = self.right_matches

    def get_watch(self):
        '''
        Function to get the possible watch type.
        Function returns nothing, but sets the following
        variables:

        self.watch_type - possible watch type

        Parameters
        ----------
        None

        Returns
        -------
        None
        '''
        watch_types = watch_type.possible_watch(self, use_left=False)
        self.right_watch_type = watch_types[0]

        watch_types = watch_type.possible_watch(self, use_left=True)
        self.left_watch_type = watch_types[0]

        if self.latitude < 0:
            self.watch_type = self.left_watch_type
        else:
            self.watch_type = self.right_watch_type

    def get_traj(self):
        '''
        Function to compute the storm slinky profile using
        the trajectory model.

        self.slinky_traj - the list containing the position vector for the updraft
        self.updraft_tilt - the updraft tilt (an angle) with respect to the horizon

        Parameters
        ----------
        None

        Returns
        -------
        None
        '''

        parcel = self.mupcl
        slinky = params.parcelTraj(self, parcel)
        if slinky == None:
            self.slinky_traj = ma.masked
            self.updraft_tilt = ma.masked
        else:
            self.slinky_traj = slinky[0]
            self.updraft_tilt = slinky[1]

    def get_PWV_loc(self):
        '''
        Function to compute the location of the current PWV with respect to
        it's sounding climatology from Bunkers.

        Parameters
        ----------
        None

        Returns
        -------
        None
        '''
        self.pwv_flag = pwv_climo(self, self.location, month=int(self.date.strftime('%m')))

    def get_indices(self):
        '''
        Function to set any additional indices that are included in the
        thermo window.

        Parameters
        ----------
        None

        Returns
        -------
        None
        '''
        self.tei = params.tei(self)
        self.esp = params.esp(self)
        self.mmp = params.mmp(self)
        self.wndg = params.wndg(self)
        self.sig_severe = params.sig_severe(self)
        self.dcape, self.dpcl_ttrace, self.dpcl_ptrace = params.dcape(self)
        self.drush = thermo.ctof(self.dpcl_ttrace[-1])
        self.mburst = params.mburst(self)

    def set_srleft(self, lm_u, lm_v):
        '''
        Sets the u and v values of the left mover supercell storm motion vector.

        Parameters
        ----------
        lm_u : number
            Left mover u-component of the storm motion vector
        lm_v : number
            Left mover v-component of the storm motion vector

        Returns
        -------
        None
        '''
        self.user_srwind = self.user_srwind[:2] + (lm_u, lm_v)
        self.get_kinematics()
        self.get_severe()

    def set_srright(self, rm_u, rm_v):
        '''
        Sets the u and v values of the right mover supercell storm motion vector.

        Parameters
        ----------
        rm_u : number
            Right mover u-component of the storm motion vector
        rm_v : number
            Right mover v-component of the storm motion vector

        Returns
        -------
        None
        '''
        self.user_srwind = (rm_u, rm_v) + self.user_srwind[2:]
        self.get_kinematics()
        self.get_severe()

    def reset_srm(self):
        '''
        Resets the storm motion vector to those found by the Bunkers algorithm

        Parameters
        ----------
        None

        Returns
        -------
        None
        '''
        self.user_srwind = self.bunkers
        self.get_kinematics()
        self.get_severe()



import requests
from sharppy._version import get_versions
import dateutil.parser as parser

## Code to check to see if this version of SHARPpy is the most recent one.

# SEE https://developer.github.com/v3/repos/releases/
url = 'https://api.github.com/repos/sharppy/SHARPpy/releases/latest'

def get_sharppy_ver():
    __version__ = get_versions()['version']
    ver = get_versions() # A dictionary with the date of the most recent commit.
    
    return ver

def get_latest_ver():
    response = requests.get(url)

    # Raise an exception if the API call fails.
    response.raise_for_status()

    data = response.json()
    return data

def get_tag_name(data):
    return data['tag_name']

def get_url(data):
    return data['html_url']

def compare_versions(data):
    # Checks to see if the publication date of the most recent Github Release
    # is less than the current version's date.
    ver = get_sharppy_ver()
    ver_date = parser.parse(ver['date'])
    latest_date = parser.parse(data['published_at'])
    return isLatest(ver_date, latest_date)

def isLatest(cur, github):
    return cur >= github

def check_latest():
    data = get_latest_ver()
    latest = compare_versions(data)
    html_url = get_url(data)
    tag_name = get_tag_name(data)
    return latest, tag_name, html_url

#   latest = check_latest()



import sharppy.io.spc_decoder as spc_decoder
import sharppy.sharptab.profile as profile
import sharppy.sharptab as tab
import numpy.testing as npt
import numpy as np

files = ['examples/data/14061619.OAX', 'examples/data/14072800.BNA']

def getProf(fname):
    dec = spc_decoder.SPCDecoder(fname)
    profs = dec.getProfiles()
    stn_id = dec.getStnId()

    all_profs = profs._profs
    prof = all_profs[''][0]
    dates = profs._dates
    prof = profile.create_profile(pres=prof.pres, hght=prof.hght, tmpc=prof.tmpc, dwpc=prof.dwpc, wspd=prof.wspd, \
                                      wdir=prof.wdir, strictQC=False, profile='convective', date=dates[0])
    return prof

profs = []
for f in files:
    profs.append(getProf(f))

def test_lapse_rate():
    prof = profs[0]
    # Values from SHARP
    assert round(tab.params.lapse_rate(prof, 0, 3000, pres=False),1) == 6.6
    assert round(tab.params.lapse_rate(prof, 3000, 6000, pres=False),1) == 8.6
    assert round(tab.params.lapse_rate(prof, 850, 500, pres=True),1) == 7.8
    assert round(tab.params.lapse_rate(prof, 700, 500, pres=True),1) == 8.8

def test_pwat():
    prof = profs[0]
    assert round(tab.params.precip_water(prof), 2) == 1.53 # Value from SHARP

def test_kindex():
    prof = profs[0]
    assert round(tab.params.k_index(prof), 0) == 37 # Value from SHARP

def test_mmp():
    prof = profs[0]
    assert round(tab.params.mmp(prof),2) == 1 # Value from SHARP

def test_wndg():
    prof = profs[0]
    assert round(tab.params.wndg(prof), 1) == 0.0 # Value from SHARP
    prof = profs[1]
    npt.assert_almost_equal(tab.params.wndg(prof),2.18739090) # Value from SHARP

def test_convT():
    prof = profs[0]
    assert round(tab.thermo.ctof(tab.params.convective_temp(prof)), 0) == 90 # Value from SHARP
    #prof = profs[1]

def test_maxT():
    prof = profs[1]
    assert round(tab.params.max_temp(prof),5) == 34.60104

def test_parcels():
    prof = profs[0]
    # Values from SHARP online
    truth_pcls = {'sfcpcl': [5765, -1, 513, -14, 613],\
                  'mlpcl': [4203, -58, 1003, -11, 2393],\
                  'mupcl': [5765, -1, 513, -14, 613],\
                  'fcstpcl': [5206, 0, 1495, -13, 1699]}

    for key in truth_pcls.keys():
        pcl = getattr(prof, key)
        returned = [round(pcl.bplus), round(pcl.bminus), round(pcl.lclhght), round(pcl.li5,0), round(pcl.lfchght,0)]
        #print(truth_pcls[key])
        #print(returned)
        bias = np.array(truth_pcls[key]) - np.array(returned)
        assert np.abs(bias).max() < 10

def test_composite_severe():
    prof = profs[0]
    assert tab.params.stp_fixed(0,0,0,0) == 0
    assert tab.params.stp_cin(0,0,0,0,0) == 0
    assert tab.params.stp_fixed(0,3000,0,0) == 0
    assert tab.params.scp(0,0,0) == 0
    assert round(tab.params.ship(prof),1) == 4.9 # Value from SHARP
    prof = profs[1]
    assert round(tab.params.ship(prof), 1) == 1.7 # Value from SHARP
    assert round(prof.right_stp_fixed,1) == 1.9 # Value from SHARP
    assert round(prof.right_stp_cin,1) == 1.9 # Value from SHARP
    
def test_tei():
    prof = profs[0]
    assert round(tab.params.tei(prof),0) == 39 # Value from SHARP
    prof = profs[1]
    assert round(tab.params.tei(prof),0) == 44 # Value from SHARP

def test_sweat():
    prof = profs[0]
    assert round(tab.params.sweat(prof)) == 622

#def test_ehi():
#    print(prof.mlpcl.bplus)
#    print(tab.params.ehi(prof.mlpcl, 0, 3000))

def test_mburst():
    prof = profs[0]
    assert tab.params.mburst(prof) == 11

def test_sherb():
    prof = profs[0]
    assert round(tab.params.sherb(prof),1) == 1.8 
    prof = profs[1]
    assert round(tab.params.sherb(prof, effective=1), 1) == 1.5 # Value from SHARP 

#test_sweat()
#test_maxT()
#test_ehi()
#test_mburst()
#test_sherb()

#test_tei()
##test_convT()
#test_wndg()
#test_mmp()
#test_parcels()
#test_kindex()
#test_pwat()    
#test_lapse_rate()
